@@ -1,15 +1,14 @@
-using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Projects;
-
 using System;
 using System.Collections.Generic;
 using System.IO;
-using Gtk;
-
 using SPath = System.IO.Path;
 
+using Gtk;
+using MonoDevelop.Core;
+using MonoDevelop.Core.Assemblies;
+using MonoDevelop.Ide;
+using MonoDevelop.Projects;
+
 namespace MonoDevelop.Autotools
 {
 	public partial class MakefileOptionPanelWidget : Gtk.Bin
@@ -18,11 +17,13 @@ namespace MonoDevelop.Autotools
 		ComboBox [] combos = null;
 		bool isDotNetProject;
 		bool loading = true;
+		Window parentDialog;
 		
-		public MakefileOptionPanelWidget (Project project, MakefileData tmpData)
+		public MakefileOptionPanelWidget (Window parentDialog, Project project, MakefileData tmpData)
 			: this ()
 		{
 			this.data = tmpData;
+			this.parentDialog = parentDialog;
 			isDotNetProject = (project is DotNetProject);
 			if (!isDotNetProject) {
 				// Disable all References combos etc for non-dotnet projects
@@ -81,7 +82,7 @@ namespace MonoDevelop.Autotools
 			//References
 			this.cbKeepRefSync.Active = data.SyncReferences;
 
-			this.entryGacRefPattern.Text = data.GacRefVar.Prefix;
+			this.entryPackageRefPattern.Text = data.PackageRefVar.Prefix;
 			this.entryAsmRefPattern.Text = data.AsmRefVar.Prefix;
 			this.entryProjectRefPattern.Text = data.ProjectRefVar.Prefix;
 			
@@ -97,6 +98,7 @@ namespace MonoDevelop.Autotools
 			this.BuildTargetName.Text = data.BuildTargetName;
 			this.ExecuteTargetName.Text = data.ExecuteTargetName;
 			this.CleanTargetName.Text = data.CleanTargetName;
+			spinProcesses.Value = data.ParallelProcesses;
 			
 			cbBuildTarget.Active = BuildTargetName.Sensitive = data.BuildTargetName != string.Empty;
 			cbRunTarget.Active = ExecuteTargetName.Sensitive = data.ExecuteTargetName != string.Empty;
@@ -137,7 +139,7 @@ namespace MonoDevelop.Autotools
 			this.Build();
 			combos = new ComboBox [7] {
 				comboFilesVar, comboDeployFilesVar, comboResourcesVar, comboOthersVar, 
-				comboGacRefVar, comboAsmRefVar, comboProjectRefVar}; 
+				comboPackageRefVar, comboAsmRefVar, comboProjectRefVar}; 
 				//comboAssemblyName, comboOutputDir};
 		}
 		
@@ -172,9 +174,9 @@ namespace MonoDevelop.Autotools
 
 			// References
 			data.SyncReferences = this.cbKeepRefSync.Active;
-			data.GacRefVar.Sync = this.cbKeepRefSync.Active;
-			data.GacRefVar.Name = GetActiveVar (comboGacRefVar);
-			data.GacRefVar.Prefix = this.entryGacRefPattern.Text.Trim ();
+			data.PackageRefVar.Sync = this.cbKeepRefSync.Active;
+			data.PackageRefVar.Name = GetActiveVar (comboPackageRefVar);
+			data.PackageRefVar.Prefix = this.entryPackageRefPattern.Text.Trim ();
 
 			data.AsmRefVar.Sync = this.cbKeepRefSync.Active;
 			data.AsmRefVar.Name = GetActiveVar (comboAsmRefVar);
@@ -193,6 +195,7 @@ namespace MonoDevelop.Autotools
 			data.BuildTargetName = this.BuildTargetName.Text.Trim ();
 			data.ExecuteTargetName = this.ExecuteTargetName.Text.Trim ();
 			data.CleanTargetName = this.CleanTargetName.Text.Trim ();
+			data.ParallelProcesses = this.spinProcesses.ValueAsInt;
 			
 			data.MessageRegexName = GetActiveVar (comboMessageType);
 			if (data.MessageRegexName == "Custom") {
@@ -213,7 +216,7 @@ namespace MonoDevelop.Autotools
 					ShowMakefileNotFoundError (e);
 					return false;
 				} catch (Exception e) {
-					MessageService.ShowException ((Window) Toplevel, e, GettextCatalog.GetString ("Specified makefile is invalid: {0}", tmpData.AbsoluteMakefileName));
+					MessageService.ShowException (parentDialog, e, GettextCatalog.GetString ("Specified makefile is invalid: {0}", tmpData.AbsoluteMakefileName));
 					return false;
 				}
 
@@ -221,16 +224,16 @@ namespace MonoDevelop.Autotools
 					!File.Exists (System.IO.Path.Combine (tmpData.AbsoluteConfigureInPath, "configure.in")) &&
 				    !File.Exists (System.IO.Path.Combine (tmpData.AbsoluteConfigureInPath, "configure.ac")))
 				{
-					MessageService.ShowError ((Window)Toplevel, GettextCatalog.GetString ("Path specified for configure.in is invalid: {0}", tmpData.RelativeConfigureInPath));
+					MessageService.ShowError (parentDialog, GettextCatalog.GetString ("Path specified for configure.in is invalid: {0}", tmpData.RelativeConfigureInPath));
 					return false;
 				}
 
 				if (tmpData.SyncReferences &&
-					(String.IsNullOrEmpty (tmpData.GacRefVar.Name) ||
+					(String.IsNullOrEmpty (tmpData.PackageRefVar.Name) ||
 					String.IsNullOrEmpty (tmpData.AsmRefVar.Name) ||
 					String.IsNullOrEmpty (tmpData.ProjectRefVar.Name))) {
 
-					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString ("'Sync References' is enabled, but one of Reference variables is not set. Please correct this."));
+					MessageService.ShowError (parentDialog, GettextCatalog.GetString ("'Sync References' is enabled, but one of Reference variables is not set. Please correct this."));
 					return false;
 				}
 			
@@ -250,14 +253,14 @@ namespace MonoDevelop.Autotools
 				try {
 					tmpData.GetErrorRegex (true);
 				} catch (Exception e) {
-					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString ("Invalid regex for Error messages: {0}", e.Message));
+					MessageService.ShowError (parentDialog, GettextCatalog.GetString ("Invalid regex for Error messages: {0}", e.Message));
 					return false;
 				}
 
 				try {
 					tmpData.GetWarningRegex (true);
 				} catch (Exception e) {
-					MessageService.ShowError ((Window) Toplevel, GettextCatalog.GetString (
+					MessageService.ShowError (parentDialog, GettextCatalog.GetString (
 						"Invalid regex for Warning messages: {0}", e.Message));
 					return false;
 				}
@@ -285,7 +288,7 @@ namespace MonoDevelop.Autotools
 		bool CheckNonEmptyFileVar (MakefileVar var, string id)
 		{
 			if (var.Sync && String.IsNullOrEmpty (var.Name.Trim ())) {
-				MessageService.ShowError ((Window) Toplevel,GettextCatalog.GetString (
+				MessageService.ShowError (parentDialog,GettextCatalog.GetString (
 					"File variable ({0}) is set for sync'ing, but no valid variable is selected. Either disable the sync'ing or select a variable name.", id));
 
 				return false;
@@ -337,7 +340,7 @@ namespace MonoDevelop.Autotools
 			SetActiveVar (comboResourcesVar, data.ResourcesVar.Name);
 			SetActiveVar (comboOthersVar, data.OthersVar.Name);
 
-			SetActiveVar (comboGacRefVar, data.GacRefVar.Name);
+			SetActiveVar (comboPackageRefVar, data.PackageRefVar.Name);
 			SetActiveVar (comboAsmRefVar, data.AsmRefVar.Name);
 			SetActiveVar (comboProjectRefVar, data.ProjectRefVar.Name);
 
@@ -446,7 +449,7 @@ namespace MonoDevelop.Autotools
 
 			string files_var = GetActiveVar (comboFilesVar);
 			string res_var = GetActiveVar (comboResourcesVar);
-			string ref_var = GetActiveVar (comboGacRefVar);
+			string ref_var = GetActiveVar (comboPackageRefVar);
 
 			string prefix;
 			foreach (string var in vars) {
@@ -535,7 +538,7 @@ namespace MonoDevelop.Autotools
 
 			cbKeepRefSync.Active = false;
 
-			entryGacRefPattern.Text = String.Empty;
+			entryPackageRefPattern.Text = String.Empty;
 			entryAsmRefPattern.Text = String.Empty;
 			entryProjectRefPattern.Text = String.Empty;
 
@@ -577,11 +580,11 @@ namespace MonoDevelop.Autotools
 			cbKeepRefSync.Active = true;
 			HandleKeepRefSyncClicked (cbKeepRefSync);
 
-			SetActiveVar (comboGacRefVar, ref_var);
+			SetActiveVar (comboPackageRefVar, ref_var);
 			SetActiveVar (comboAsmRefVar, ref_var);
 			SetActiveVar (comboProjectRefVar, ref_var);
 
-			entryGacRefPattern.Text = prefix;
+			entryPackageRefPattern.Text = prefix;
 			entryAsmRefPattern.Text = prefix;
 			entryProjectRefPattern.Text = prefix;
 		}
@@ -664,9 +667,9 @@ namespace MonoDevelop.Autotools
 			this.lblCol5.Sensitive = state;
 			this.lblCol6.Sensitive = state;
 			
-			this.lblGacRef.Sensitive = state;
-			this.comboGacRefVar.Sensitive = state;
-			this.entryGacRefPattern.Sensitive = state;
+			this.lblPackageRef.Sensitive = state;
+			this.comboPackageRefVar.Sensitive = state;
+			this.entryPackageRefPattern.Sensitive = state;
 
 			this.lblAsmRef.Sensitive = state;
 			this.comboAsmRefVar.Sensitive = state;
@@ -740,14 +743,14 @@ namespace MonoDevelop.Autotools
 					ShowMakefileNotFoundError (e);
 			} catch (Exception e) {
 				if (showError)
-					MessageService.ShowException ((Window) this.Toplevel,e,
+					MessageService.ShowException (parentDialog,e,
 						GettextCatalog.GetString ("Error while trying to read the specified Makefile"));
 				return null;
 			}
 
 			if (vars != null && vars.Count == 0) {
 				if (showError)
-					MessageService.ShowError ((Window) this.Toplevel, 
+					MessageService.ShowError (parentDialog, 
 						GettextCatalog.GetString ("No variables found in the selected Makefile"));
 				return null;
 			}
@@ -888,15 +891,15 @@ namespace MonoDevelop.Autotools
 			}
 		}
 
-		protected virtual void OnComboGacRefVarChanged (object sender, System.EventArgs e)
+		protected virtual void OnComboPackageRefVarChanged (object sender, System.EventArgs e)
 		{
-			HandleComboGacRefVarChanged ((ComboBox) sender);
+			HandleComboPackageRefVarChanged ((ComboBox) sender);
 		}
 
-		void HandleComboGacRefVarChanged (ComboBox cb)
+		void HandleComboPackageRefVarChanged (ComboBox cb)
 		{
 			string active = GetActiveVar (cb);
-			entryGacRefPattern.Text = GuessRefPrefix (data.Makefile.GetListVariable (active));
+			entryPackageRefPattern.Text = GuessRefPrefix (data.Makefile.GetListVariable (active));
 		}
 
 		protected virtual void OnComboAsmRefVarChanged (object sender, System.EventArgs e)
@@ -934,7 +937,7 @@ namespace MonoDevelop.Autotools
 
 		void ShowMakefileNotFoundError (Exception e)
 		{
-				MessageService.ShowException ((Window) this.Toplevel, 
+				MessageService.ShowException (parentDialog, 
 			                                  e,
 			                                  GettextCatalog.GetString ("Unable to find the specified Makefile. You need to specify the path to an existing Makefile for use with the 'Makefile Integration' feature."));
 		}
@@ -951,20 +954,32 @@ namespace MonoDevelop.Autotools
 			if (prefix.Length > 0)
 				return true;
 
+			DotNetProject dnp = data.OwnerProject as DotNetProject;
+			if (dnp == null)
+				return false;
+			
 			// 'core' here simply means any assemblies in the gac
-			bool has_core_or_pkgref = false;
 			foreach (string file in list) {
-				if (MakefileData.CorePackageAssemblyNames.ContainsKey (file) || IsPkgRef (file)) {
-					has_core_or_pkgref = true;
-					continue;
+				try {
+					string fullName = dnp.AssemblyContext.GetAssemblyFullName (file, dnp.TargetFramework);
+					if (fullName != null) {
+						SystemAssembly asm = dnp.AssemblyContext.GetAssemblyFromFullName (fullName, null, dnp.TargetFramework);
+						if (asm != null && asm.Package.IsGacPackage)
+							return true;
+					}
+				} catch {
+					// Ignore
 				}
+					
+				if (IsPkgRef (file))
+					return true;
 
 				// invalid if any entry isn't one of core/variable/dll/pkgrefs
 				if (!IsVariable (file) && !IsDll (file))
 					return false;
 			}
 
-			return has_core_or_pkgref;
+			return false;
 		}
 
 		bool CheckRes (List<string> list, out string prefix)
