@@ -31,6 +31,7 @@ using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Deployment;
+using MonoDevelop.Core.Assemblies;
 
 using Mono.Unix.Native;
 
@@ -44,11 +45,18 @@ namespace MonoDevelop.Autotools
 
 		AutotoolsContext context;
 		bool generateAutotools;
+		
+		List<Switch> switchs;
 
 		public SolutionDeployer (bool generateAutotools)
 		{
 			this.generateAutotools = generateAutotools;
 		}
+		
+		public void AddSwitches (IEnumerable<Switch> switches)
+		{
+			switchs = new List<Switch> (switches);
+		}
 
 		public bool HasGeneratedFiles (Solution solution)
 		{
@@ -87,6 +95,7 @@ namespace MonoDevelop.Autotools
 				
 				context = new AutotoolsContext ( ctx, solution_dir, configs, mt );
 				context.TargetSolution = solution;
+				context.Switches = switchs;
 				
 				IMakefileHandler handler = AutotoolsContext.GetMakefileHandler (solution.RootFolder, mt);
 				if (handler == null)
@@ -203,7 +212,7 @@ namespace MonoDevelop.Autotools
 			{
 				monitor.EndTask ();
 			}
-			monitor.ReportSuccess ( GettextCatalog.GetString ( "Solution was succesfully deployed" ) );
+			monitor.ReportSuccess (GettextCatalog.GetString ("Solution was successfully deployed."));
 			return true;
 		}
 
@@ -297,6 +306,20 @@ namespace MonoDevelop.Autotools
 				config_options.Append ("fi\n");
 			}
 
+			// Add specific user switch
+			if (switchs != null) {
+				foreach (Switch s in switchs) {
+					string name = s.SwitchName.ToLowerInvariant ();
+					
+					config_options.AppendLine (string.Format (@"AC_ARG_ENABLE({0},
+	AC_HELP_STRING([--enable-{0}],
+		[{1}]),
+		enable_{2}=yes, enable_{2}=no)
+AM_CONDITIONAL(ENABLE_{3}, test x$enable_{2} = xyes)", 
+				                       name, s.HelpStr, name.Replace ('-', '_'), name.Replace ('-', '_').ToUpperInvariant ()));
+				}
+			}
+			
 			templateEngine.Variables ["CONFIG_OPTIONS"] = config_options.ToString();
 
 			// build compiler checks
@@ -319,6 +342,7 @@ namespace MonoDevelop.Autotools
 				if (PlatformID.Unix != Environment.OSVersion.Platform)
 					tmpmf = tmpmf.Replace("\\","/");
 
+				AutotoolsContext.CheckSpaces (tmpmf);
 				configFiles.Append(FileService.NormalizeRelativePath (tmpmf));
 				configFiles.Append("\n");
 			}
