@@ -37,9 +37,10 @@ using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Text;
 using MonoDevelop.Debugger.Viewers;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.Semantics;
 
 /*
  * Some places we should be doing some error handling we used to toss
@@ -63,11 +64,15 @@ namespace MonoDevelop.Debugger
 		static IConsole console;
 		static DebugExecutionHandlerFactory executionHandlerFactory;
 		
+		static string oldLayout;
+		
 		static DebuggerEngine currentEngine;
 		static DebuggerSession session;
 		static Backtrace currentBacktrace;
 		static int currentFrame;
-
+		
+		static ExceptionCaughtDialog exceptionDialog;
+		
 		static BusyEvaluatorDialog busyDialog;
 		static bool isBusy;
 		static StatusBarIcon busyStatusIcon;
@@ -94,6 +99,8 @@ namespace MonoDevelop.Debugger
 				IdeApp.Workspace.LoadingUserPreferences += OnLoadUserPrefs;
 				IdeApp.Workspace.LastWorkspaceItemClosed += OnSolutionClosed;
 				busyDialog = new BusyEvaluatorDialog ();
+				busyDialog.TransientFor = MessageService.RootWindow;
+				busyDialog.DestroyWithParent = true;
 			};
 			AddinManager.AddExtensionNodeHandler (FactoriesPath, delegate {
 				// Regresh the engine list
@@ -192,13 +199,9 @@ namespace MonoDevelop.Debugger
 		
 		public static void ShowValueVisualizer (ObjectValue val)
 		{
-			ValueVisualizerDialog dlg = new ValueVisualizerDialog ();
-			try {
-				dlg.Show (val);
-				dlg.Run ();
-			} finally {
-				dlg.Destroy ();
-			}
+			var dlg = new ValueVisualizerDialog ();
+			dlg.Show (val);
+			MessageService.ShowCustomDialog (dlg);
 		}
 		
 		public static bool ShowBreakpointProperties (Breakpoint bp, bool editNew)
@@ -281,10 +284,18 @@ namespace MonoDevelop.Debugger
 		
 		public static void ShowExceptionCaughtDialog ()
 		{
-			ExceptionInfo val = CurrentFrame.GetException ();
+			EvaluationOptions ops = session.EvaluationOptions.Clone ();
+			ops.EllipsizeStrings = false;
+			
+			ExceptionInfo val = CurrentFrame.GetException (ops);
 			if (val != null) {
-				ExceptionCaughtDialog dlg = new ExceptionCaughtDialog (val);
-				dlg.Show ();
+				exceptionDialog = new ExceptionCaughtDialog (val);
+				exceptionDialog.TransientFor = IdeApp.Workbench.RootWindow;
+				MessageService.PlaceDialog (exceptionDialog, IdeApp.Workbench.RootWindow);
+				exceptionDialog.Destroyed += (o, args) => {
+					exceptionDialog = null;
+				};
+				exceptionDialog.Show ();
 			}
 		}
 
@@ -301,27 +312,46 @@ namespace MonoDevelop.Debugger
 			session.TargetStarted += OnStarted;
 			session.OutputWriter = OutputWriter;
 			session.LogWriter = LogWriter;
-			session.ExceptionHandler = ExceptionHandler;
 			session.BusyStateChanged += OnBusyStateChanged;
 			session.TypeResolverHandler = ResolveType;
 			session.BreakpointTraceHandler = BreakpointTraceHandler;
 			session.GetExpressionEvaluator = OnGetExpressionEvaluator;
+			session.ConnectionDialogCreator = delegate {
+				return new GtkConnectionDialog ();
+			};
 
 			console.CancelRequested += OnCancelRequested;
 			
-			if (DebugSessionStarted != null)
-				DebugSessionStarted (null, EventArgs.Empty);
+			DispatchService.GuiDispatch (delegate {
+				if (DebugSessionStarted != null)
+					DebugSessionStarted (null, EventArgs.Empty);
+				NotifyLocationChanged ();
+			});
 
-			NotifyLocationChanged ();
 		}
 		
 		static void Cleanup ()
 		{
+			if (oldLayout != null) {
+				string layout = oldLayout;
+				oldLayout = null;
+				// Dispatch asynchronously to avoid start/stop races
+				DispatchService.GuiSyncDispatch (delegate {
+					if (IdeApp.Workbench.CurrentLayout == "Debug")
+						IdeApp.Workbench.CurrentLayout = layout;
+				});
+			}
+			
 			currentBacktrace = null;
 			
 			if (!IsDebugging)
 				return;
-
+			
+			if (exceptionDialog != null) {
+				exceptionDialog.Destroy ();
+				exceptionDialog = null;
+			}
+			
 			if (busyStatusIcon != null) {
 				busyStatusIcon.Dispose ();
 				busyStatusIcon = null;
@@ -331,7 +361,6 @@ namespace MonoDevelop.Debugger
 			session.TargetStarted -= OnStarted;
 			session.OutputWriter = null;
 			session.LogWriter = null;
-			session.ExceptionHandler = null;
 			session.BusyStateChanged -= OnBusyStateChanged;
 			session.TypeResolverHandler = null;
 			session.BreakpointTraceHandler = null;
@@ -341,9 +370,11 @@ namespace MonoDevelop.Debugger
 			// Dispose the session at the end, since it may take a while.
 			DebuggerSession oldSession = session;
 			session = null;
-
-			if (StoppedEvent != null)
-				StoppedEvent (null, new EventArgs ());
+			
+			DispatchService.GuiDispatch (delegate {
+				if (StoppedEvent != null)
+					StoppedEvent (null, new EventArgs ());
+			});
 			
 			if (console != null) {
 				console.Dispose ();
@@ -374,6 +405,12 @@ namespace MonoDevelop.Debugger
 			}
 		}
 
+		public static bool IsPaused {
+			get {
+				return IsDebugging && !IsRunning && currentBacktrace != null;
+			}
+		}
+
 		public static void Pause ()
 		{
 			session.Stop ();
@@ -398,6 +435,7 @@ namespace MonoDevelop.Debugger
 		{
 			currentEngine = debugger;
 			session = debugger.CreateSession ();
+			session.ExceptionHandler = ExceptionHandler;
 			IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetRunProgressMonitor ();
 			console = monitor as IConsole;
 			SetupSession ();
@@ -410,23 +448,26 @@ namespace MonoDevelop.Debugger
 		
 		public static DebuggerSessionOptions GetUserOptions ()
 		{
-			EvaluationOptions eops = EvaluationOptions.DefaultOptions;
-			eops.AllowTargetInvoke = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", true);
-			eops.AllowToStringCalls = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", true);
-			eops.EvaluationTimeout = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", 2500);
-			eops.FlattenHierarchy = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.FlattenHierarchy", false);
-			eops.GroupPrivateMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupPrivateMembers", true);
-			eops.GroupStaticMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupStaticMembers", true);
-			eops.MemberEvaluationTimeout = eops.EvaluationTimeout * 2;
+			EvaluationOptions eval = EvaluationOptions.DefaultOptions;
+			eval.AllowTargetInvoke = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", true);
+			eval.AllowToStringCalls = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", true);
+			eval.EvaluationTimeout = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", 2500);
+			eval.FlattenHierarchy = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.FlattenHierarchy", false);
+			eval.GroupPrivateMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupPrivateMembers", true);
+			eval.GroupStaticMembers = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.GroupStaticMembers", true);
+			eval.MemberEvaluationTimeout = eval.EvaluationTimeout * 2;
 			return new DebuggerSessionOptions () {
+				StepOverPropertiesAndOperators = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.StepOverPropertiesAndOperators", true),
 				ProjectAssembliesOnly = PropertyService.Get ("MonoDevelop.Debugger.DebuggingService.ProjectAssembliesOnly", true),
-				EvaluationOptions = eops,
+				EvaluationOptions = eval,
 			};
 		}
 		
 		public static void SetUserOptions (DebuggerSessionOptions options)
 		{
+			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.StepOverPropertiesAndOperators", options.StepOverPropertiesAndOperators);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.ProjectAssembliesOnly", options.ProjectAssembliesOnly);
+			
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.AllowTargetInvoke", options.EvaluationOptions.AllowTargetInvoke);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.AllowToStringCalls", options.EvaluationOptions.AllowToStringCalls);
 			PropertyService.Set ("MonoDevelop.Debugger.DebuggingService.EvaluationTimeout", options.EvaluationOptions.EvaluationTimeout);
@@ -455,16 +496,31 @@ namespace MonoDevelop.Debugger
 					throw new InvalidOperationException ("Unsupported command: " + cmd);
 			}
 			
+			if (session != null)
+				throw new InvalidOperationException ("A debugger session is already started");
+
 			DebuggerStartInfo startInfo = factory.CreateDebuggerStartInfo (cmd);
 			startInfo.UseExternalConsole = c is ExternalConsole;
 			startInfo.CloseExternalConsoleOnExit = c.CloseOnDispose;
 			currentEngine = factory;
 			session = factory.CreateSession ();
-			session.Initialize ();
-			console = c;
+			session.ExceptionHandler = ExceptionHandler;
+			
+			// When using an external console, create a new internal console which will be used
+			// to show the debugger log
+			if (startInfo.UseExternalConsole)
+				console = (IConsole) IdeApp.Workbench.ProgressMonitors.GetRunProgressMonitor ();
+			else
+				console = c;
 			
 			SetupSession ();
-
+			
+			// Dispatch synchronously to avoid start/stop races
+			DispatchService.GuiSyncDispatch (delegate {
+				oldLayout = IdeApp.Workbench.CurrentLayout;
+				IdeApp.Workbench.CurrentLayout = "Debug";
+			});
+			
 			try {
 				session.Run (startInfo, GetUserOptions ());
 			} catch {
@@ -475,6 +531,7 @@ namespace MonoDevelop.Debugger
 		
 		static bool ExceptionHandler (Exception ex)
 		{
+			LoggingService.LogError ("Error in debugger", ex);
 			Gtk.Application.Invoke (delegate {
 				if (ex is DebuggerException)
 					MessageService.ShowError (ex.Message);
@@ -511,7 +568,7 @@ namespace MonoDevelop.Debugger
 						busyStatusIcon.SetAlertMode (100);
 						busyStatusIcon.ToolTip = GettextCatalog.GetString ("The Debugger is waiting for an expression evaluation to finish.");
 						busyStatusIcon.EventBox.ButtonPressEvent += delegate {
-							busyDialog.Show ();
+							MessageService.PlaceDialog (busyDialog, MessageService.RootWindow);
 						};
 					}
 				} else {
@@ -526,7 +583,7 @@ namespace MonoDevelop.Debugger
 		static bool CheckIsBusy ()
 		{
 			if (isBusy && !busyDialog.Visible)
-				busyDialog.Show ();
+				MessageService.PlaceDialog (busyDialog, MessageService.RootWindow);
 			return isBusy;
 		}
 		
@@ -545,30 +602,25 @@ namespace MonoDevelop.Debugger
 		static void OnTargetEvent (object sender, TargetEventArgs args)
 		{
 			try {
-				Console.WriteLine ("OnTargetEvent, type - {0}", args.Type);
-				if (args.Type != TargetEventType.TargetExited) {
-					SetCurrentBacktrace (args.Backtrace);
-				}
-
 				switch (args.Type) {
 					case TargetEventType.TargetExited:
 						Cleanup ();
 						break;
 					case TargetEventType.TargetSignaled:
 					case TargetEventType.TargetStopped:
-					case TargetEventType.TargetRunning:
 					case TargetEventType.TargetHitBreakpoint:
 					case TargetEventType.TargetInterrupted:
 					case TargetEventType.UnhandledException:
 					case TargetEventType.ExceptionThrown:
+						SetCurrentBacktrace (args.Backtrace);
 						NotifyPaused ();
 						NotifyException (args);
 						break;
 					default:
 						break;
 				}
-			} catch (Exception e) {
-				Console.WriteLine ("OnTargetEvent, {0}", e.ToString ());
+			} catch (Exception ex) {
+				LoggingService.LogError ("Error handling debugger target event", ex);
 			}
 		}
 		
@@ -585,7 +637,7 @@ namespace MonoDevelop.Debugger
 				if (PausedEvent != null)
 					PausedEvent (null, EventArgs.Empty);
 				NotifyLocationChanged ();
-				IdeApp.Workbench.Present ();
+				IdeApp.Workbench.GrabDesktopFocus ();
 			});
 		}
 		
@@ -733,8 +785,8 @@ namespace MonoDevelop.Debugger
 		{
 			if (currentBacktrace != null) {
 				var sf = GetCurrentVisibleFrame ();
-				if (!string.IsNullOrEmpty (sf.SourceLocation.Filename) && System.IO.File.Exists (sf.SourceLocation.Filename) && sf.SourceLocation.Line != -1) {
-					Document document = IdeApp.Workbench.OpenDocument (sf.SourceLocation.Filename, sf.SourceLocation.Line, 1, true, false);
+				if (!string.IsNullOrEmpty (sf.SourceLocation.FileName) && System.IO.File.Exists (sf.SourceLocation.FileName) && sf.SourceLocation.Line != -1) {
+					Document document = IdeApp.Workbench.OpenDocument (sf.SourceLocation.FileName, sf.SourceLocation.Line, 1, OpenDocumentOptions.Debugger);
 					OnDisableConditionalCompilation (new DocumentEventArgs (document));
 				}
 			}
@@ -796,14 +848,14 @@ namespace MonoDevelop.Debugger
 		
 		static void OnLineCountChanged (object ob, LineCountEventArgs a)
 		{
-			List<Breakpoint> bps = new List<Breakpoint> (breakpoints.GetBreakpoints ());
-			foreach (Breakpoint bp in bps) {
+			foreach (Breakpoint bp in breakpoints.GetBreakpoints ()) {
 				if (bp.FileName == a.TextFile.Name) {
 					if (bp.Line > a.LineNumber) {
-						// If the line that has the breakpoint is deleted, delete the breakpoint
-						breakpoints.Remove (bp);
+						// If the line that has the breakpoint is deleted, delete the breakpoint, otherwise update the line #.
 						if (bp.Line + a.LineCount >= a.LineNumber)
-							breakpoints.Add (bp.FileName, bp.Line + a.LineCount);
+							breakpoints.UpdateBreakpointLine (bp, bp.Line + a.LineCount);
+						else
+							breakpoints.Remove (bp);
 					}
 					else if (bp.Line == a.LineNumber && a.LineCount < 0)
 						breakpoints.Remove (bp);
@@ -837,17 +889,17 @@ namespace MonoDevelop.Debugger
 		
 		static string ResolveType (string identifier, SourceLocation location)
 		{
-			Document doc = IdeApp.Workbench.GetDocument (location.Filename);
+			Document doc = IdeApp.Workbench.GetDocument (location.FileName);
 			if (doc != null) {
 				ITextEditorResolver textEditorResolver = doc.GetContent <ITextEditorResolver> ();
 				if (textEditorResolver != null) {
-					ResolveResult rr = textEditorResolver.GetLanguageItem (doc.TextEditor.GetPositionFromLineColumn (location.Line, 1), identifier);
-					NamespaceResolveResult ns = rr as NamespaceResolveResult;
+					var rr = textEditorResolver.GetLanguageItem (doc.Editor.Document.LocationToOffset (location.Line, 1), identifier);
+					var ns = rr as NamespaceResolveResult;
 					if (ns != null)
-						return ns.Namespace;
-					MemberResolveResult result = rr as MemberResolveResult;
-					if (result != null && (result.ResolvedMember == null || result.ResolvedMember is IType) && result.ResolvedType != null)
-						return result.ResolvedType.FullName;
+						return ns.NamespaceName;
+					var result = rr as TypeResolveResult;
+					if (result != null && !result.IsError)
+						return result.Type.FullName;
 				}
 			}
 			return null;
@@ -907,4 +959,72 @@ namespace MonoDevelop.Debugger
 			return h.Execute (command, console);
 		}
 	}
+	
+	internal class GtkConnectionDialog : IConnectionDialog
+	{
+		bool disposed;
+		System.Threading.CancellationTokenSource cts;
+		
+		public event EventHandler UserCancelled;
+		
+		string DefaultListenMessage {
+			get { return GettextCatalog.GetString ("Waiting for debugger to connect..."); }
+		}
+
+		public void SetMessage (DebuggerStartInfo dsi, string message, bool listening, int attemptNumber)
+		{
+			//FIXME: we don't support changing the message
+			if (disposed || cts != null)
+				return;
+			
+			cts = new System.Threading.CancellationTokenSource ();
+			
+			//MessageService is threadsafe but we want this to be async
+			Gtk.Application.Invoke (delegate {
+				RunDialog (message);
+			});
+		}
+		
+		void RunDialog (string message)
+		{
+			if (disposed)
+				return;
+			
+			string title;
+			
+			if (message == null) {
+				title = GettextCatalog.GetString ("Waiting for debugger");
+			} else {
+				message = message.Trim ();
+				int i = message.IndexOfAny (new char[] { '\n', '\r' });
+				if (i > 0) {
+					title = message.Substring (0, i).Trim ();
+					message = message.Substring (i).Trim ();
+				} else {
+					title = message;
+					message = null;
+				}
+			}
+
+			var gm = new GenericMessage (title, message, cts.Token);
+			gm.Buttons.Add (AlertButton.Cancel);
+			gm.DefaultButton = 0;
+			MessageService.GenericAlert (gm);
+			cts = null;
+			
+			if (!disposed && UserCancelled != null) {
+				UserCancelled (null, null);
+			}
+		}
+
+		public void Dispose ()
+		{
+			if (disposed)
+				return;
+			disposed = true;
+			var c = cts;
+			if (c != null)
+				c.Cancel ();
+		}
+	}
 }
