@@ -39,8 +39,7 @@ using MonoDevelop.Components.Commands;
 
 namespace MonoDevelop.SourceEditor
 {
-	
-	partial class SearchAndReplaceWidget : Bin
+	public partial class SearchAndReplaceWidget : Bin
 	{
 		const char historySeparator = '\n';
 		const int  historyLimit = 20;
@@ -50,8 +49,20 @@ namespace MonoDevelop.SourceEditor
 		internal static string searchPattern = String.Empty;
 		internal static string replacePattern = String.Empty;
 
-		SourceEditorWidget widget;
-		
+		public TextSegment SelectionSegment {
+			get;
+			set;
+		}
+
+		public bool IsInSelectionSearchMode {
+			get;
+			set;
+		}
+
+		readonly TextEditor textEditor;
+		readonly Widget frame;
+		readonly TextEditorContainer textEditorContainer;
+
 		bool isReplaceMode = true;
 		Widget [] replaceWidgets;
 		
@@ -91,54 +102,55 @@ namespace MonoDevelop.SourceEditor
 			}
 		}
 		
-		Widget container;
 		void HandleViewTextEditorhandleSizeAllocated (object o, SizeAllocatedArgs args)
 		{
-			int newX = widget.TextEditor.Allocation.Width - this.Allocation.Width - 8;
-			TextEditorContainer.EditorContainerChild containerChild = ((Mono.TextEditor.TextEditorContainer.EditorContainerChild)widget.TextEditorContainer[container]);
+			if (frame == null || textEditor == null)
+				return;
+			int newX = textEditor.Allocation.Width - Allocation.Width - 8;
+			TextEditorContainer.EditorContainerChild containerChild = ((TextEditorContainer.EditorContainerChild)textEditorContainer [frame]);
 			if (newX != containerChild.X) {
-				this.searchEntry.WidthRequest = widget.Vbox.Allocation.Width / 3;
+				searchEntry.WidthRequest = textEditor.Allocation.Width / 3;
 				containerChild.X = newX;
-				widget.TextEditorContainer.QueueResize ();
+				textEditorContainer.QueueResize ();
 			}
 		}
 		
-		public SearchAndReplaceWidget (SourceEditorWidget widget, Widget container)
+		public SearchAndReplaceWidget (TextEditor textEditor, Widget frame)
 		{
-			this.container = container;
-			widget.TextEditorContainer.SizeAllocated += HandleViewTextEditorhandleSizeAllocated;
-			widget.TextEditor.TextViewMargin.SearchRegionsUpdated += HandleWidgetTextEditorTextViewMarginSearchRegionsUpdated;
-			widget.TextEditor.Caret.PositionChanged += HandleWidgetTextEditorCaretPositionChanged;
-			this.SizeAllocated += HandleViewTextEditorhandleSizeAllocated;
-			this.Name = "SearchAndReplaceWidget";
-			this.Events = Gdk.EventMask.AllEventsMask;
-			widget.DisableAutomaticSearchPatternCaseMatch = false;
-			Build();
-			this.buttonReplace.TooltipText = GettextCatalog.GetString ("Replace");
-			this.buttonSearchForward.TooltipText = GettextCatalog.GetString ("Find next");
-			this.buttonSearchBackward.TooltipText = GettextCatalog.GetString ("Find previous");
-			this.buttonSearchMode.TooltipText = GettextCatalog.GetString ("Toggle between search and replace mode");
-			this.searchEntry.Ready = true;
-			this.searchEntry.Visible = true;
-			this.searchEntry.WidthRequest = widget.Vbox.Allocation.Width / 3;
-			this.searchEntry.ForceFilterButtonVisible = true;
+			this.textEditor = textEditor;
+			textEditorContainer = textEditor.Parent as TextEditorContainer;
+			this.frame = frame;
+			textEditorContainer.SizeAllocated += HandleViewTextEditorhandleSizeAllocated;
+			textEditor.TextViewMargin.SearchRegionsUpdated += HandleWidgetTextEditorTextViewMarginSearchRegionsUpdated;
+			textEditor.Caret.PositionChanged += HandleWidgetTextEditorCaretPositionChanged;
+			SizeAllocated += HandleViewTextEditorhandleSizeAllocated;
+			Name = "SearchAndReplaceWidget";
+			Events = Gdk.EventMask.AllEventsMask;
+			DisableAutomaticSearchPatternCaseMatch = false;
+			Build ();
+			buttonReplace.TooltipText = GettextCatalog.GetString ("Replace");
+			buttonSearchForward.TooltipText = GettextCatalog.GetString ("Find next");
+			buttonSearchBackward.TooltipText = GettextCatalog.GetString ("Find previous");
+			buttonSearchMode.TooltipText = GettextCatalog.GetString ("Toggle between search and replace mode");
+			searchEntry.Ready = true;
+			searchEntry.Visible = true;
+			searchEntry.WidthRequest = textEditor.Allocation.Width / 3;
+			searchEntry.ForceFilterButtonVisible = true;
 			replaceWidgets = new Widget [] {
-		//		labelReplace,
+			//		labelReplace,
 				entryReplace,
 				buttonReplace,
 				buttonReplaceAll
 			};
 			
-			this.FocusChain = new Widget [] {
-				this.searchEntry,
-				this.buttonSearchForward,
-				this.buttonSearchBackward,
+			FocusChain = new Widget [] {
+				searchEntry,
+				buttonSearchForward,
+				buttonSearchBackward,
 				entryReplace,
 				buttonReplace,
 				buttonReplaceAll
 			};
-			
-			this.widget = widget;
 			FilterHistory (seachHistoryProperty);
 			FilterHistory (replaceHistoryProperty);
 			//HACK: GTK rendering issue on Mac, images don't repaint unless we put them in visible eventboxes
@@ -149,10 +161,10 @@ namespace MonoDevelop.SourceEditor
 				}
 			}
 
-			if (String.IsNullOrEmpty (widget.TextEditor.SearchPattern)) {
-				widget.TextEditor.SearchPattern = searchPattern;
-			} else if (widget.TextEditor.SearchPattern != searchPattern) {
-				searchPattern = widget.TextEditor.SearchPattern;
+			if (String.IsNullOrEmpty (textEditor.SearchPattern)) {
+				textEditor.SearchPattern = searchPattern;
+			} else if (textEditor.SearchPattern != searchPattern) {
+				searchPattern = textEditor.SearchPattern;
 				//FireSearchPatternChanged ();
 			}
 			UpdateSearchPattern ();
@@ -160,20 +172,20 @@ namespace MonoDevelop.SourceEditor
 			//searchEntry.Model = searchHistory;
 			
 			searchEntry.Entry.KeyReleaseEvent += delegate {
-				widget.CheckSearchPatternCasing (SearchPattern);
+				CheckSearchPatternCasing (SearchPattern);
 /*				widget.SetSearchPattern (SearchPattern);
 				searchPattern = SearchPattern;
 				UpdateSearchEntry ();*/
 			};
 			
 			searchEntry.Entry.Changed += delegate {
-				widget.SetSearchPattern (SearchPattern);
+				SetSearchPattern (SearchPattern);
 				string oldPattern = searchPattern;
 				searchPattern = SearchPattern;
 				if (oldPattern != searchPattern)
 					UpdateSearchEntry ();
 				var history = GetHistory (seachHistoryProperty);
-				if (history.Count > 0 && history[0] == oldPattern) {
+				if (history.Count > 0 && history [0] == oldPattern) {
 					ChangeHistory (seachHistoryProperty, searchPattern);
 				} else {
 					UpdateSearchHistory (searchPattern);
@@ -187,12 +199,12 @@ namespace MonoDevelop.SourceEditor
 			foreach (Gtk.Widget child in Children) {
 				child.KeyPressEvent += delegate (object sender, Gtk.KeyPressEventArgs args) {
 					if (args.Event.Key == Gdk.Key.Escape)
-						widget.RemoveSearchWidget ();
+						RemoveSearchWidget ();
 				};
 			}
 			
 			closeButton.Clicked += delegate {
-				widget.RemoveSearchWidget ();
+				RemoveSearchWidget ();
 			};
 			
 			buttonSearchMode.Clicked += delegate {
@@ -211,17 +223,17 @@ namespace MonoDevelop.SourceEditor
 			
 			searchEntry.Entry.Activated += delegate {
 				UpdateSearchHistory (SearchPattern);
-				widget.FindNext (false);
+				FindNext (textEditor);
 			};
 			
 			buttonSearchForward.Clicked += delegate {
 				UpdateSearchHistory (SearchPattern);
-				widget.FindNext (false);
+				FindNext (textEditor);
 			};
 			
 			buttonSearchBackward.Clicked += delegate {
 				UpdateSearchHistory (SearchPattern);
-				widget.FindPrevious (false);
+				FindPrevious (textEditor);
 			};
 			
 //			optionsButton.Label = MonoDevelop.Core.GettextCatalog.GetString ("Options");
@@ -237,20 +249,20 @@ namespace MonoDevelop.SourceEditor
 			entryReplace.Activated += delegate {
 				UpdateSearchHistory (SearchPattern);
 				UpdateReplaceHistory (ReplacePattern);
-				widget.Replace ();
+				Replace ();
 				entryReplace.GrabFocus ();
 			};
 			
 			buttonReplace.Clicked += delegate {
 				UpdateSearchHistory (SearchPattern);
 				UpdateReplaceHistory (ReplacePattern);
-				widget.Replace ();
+				Replace ();
 			};
 			
 			buttonReplaceAll.Clicked += delegate {
 				UpdateSearchHistory (SearchPattern);
 				UpdateReplaceHistory (ReplacePattern);
-				widget.ReplaceAll ();
+				ReplaceAll ();
 			};
 			
 			buttonSearchForward.KeyPressEvent += OnNavigateKeyPressEvent;
@@ -265,8 +277,63 @@ namespace MonoDevelop.SourceEditor
 			resultInformLabel.Xpad = 2;
 			resultInformLabel.Show ();
 			searchEntry.FilterButtonPixbuf = new Gdk.Pixbuf (typeof(SearchAndReplaceWidget).Assembly, "searchoptions.png");
+
+			if (textEditor.IsSomethingSelected) {
+				if (textEditor.MainSelection.MinLine == textEditor.MainSelection.MaxLine) {
+					SetSearchPattern ();
+				} else {
+					IsInSelectionSearchMode = true;
+					SelectionSegment = textEditor.SelectionRange;
+				}
+			}
+			SetSearchPattern (searchPattern);
+			textEditor.HighlightSearchPattern = true;
+			textEditor.TextViewMargin.RefreshSearchMarker ();
+			Console.WriteLine ("readonly" + textEditor.Document.ReadOnly);
+			if (textEditor.Document.ReadOnly) {
+				buttonSearchMode.Visible = false;
+				IsReplaceMode = false;
+			}
+			
 		}
 
+		public bool DisableAutomaticSearchPatternCaseMatch {
+			get;
+			set;
+		}
+		
+		internal void CheckSearchPatternCasing (string searchPattern)
+		{
+			if (!DisableAutomaticSearchPatternCaseMatch && PropertyService.Get ("AutoSetPatternCasing", true)) {
+				IsCaseSensitive = searchPattern.Any (ch => Char.IsUpper (ch));
+				SetSearchOptions ();
+			}
+		}
+		
+		void SetSearchOptions ()
+		{
+			if (SearchAndReplaceWidget.SearchEngine == SearchAndReplaceWidget.DefaultSearchEngine) {
+				if (!(textEditor.SearchEngine is BasicSearchEngine))
+					textEditor.SearchEngine = new BasicSearchEngine ();
+			} else {
+				if (!(textEditor.SearchEngine is RegexSearchEngine))
+					textEditor.SearchEngine = new RegexSearchEngine ();
+			}
+			textEditor.IsCaseSensitive = IsCaseSensitive;
+			textEditor.IsWholeWordOnly = SearchAndReplaceWidget.IsWholeWordOnly;
+			textEditor.SearchRegion = IsInSelectionSearchMode ? SelectionSegment : TextSegment.Invalid;
+			string error;
+			string pattern = SearchPattern;
+			
+			bool valid = textEditor.SearchEngine.IsValidPattern (pattern, out error);
+			
+			if (valid) {
+				textEditor.SearchPattern = pattern;
+			}
+			textEditor.QueueDraw ();
+		}
+		
+
 		void HandleSearchEntryhandleRequestMenu (object sender, EventArgs e)
 		{
 			if (searchEntry.Menu != null)
@@ -274,7 +341,7 @@ namespace MonoDevelop.SourceEditor
 			
 			searchEntry.Menu = new Menu ();
 			
-			CheckMenuItem caseSensitive = new CheckMenuItem (MonoDevelop.Core.GettextCatalog.GetString ("_Case sensitive"));
+			CheckMenuItem caseSensitive = new CheckMenuItem (GettextCatalog.GetString ("_Case sensitive"));
 			caseSensitive.Active = IsCaseSensitive;
 			caseSensitive.DrawAsRadio = false;
 			caseSensitive.Toggled += delegate {
@@ -283,7 +350,7 @@ namespace MonoDevelop.SourceEditor
 			};
 			searchEntry.Menu.Add (caseSensitive);
 			
-			CheckMenuItem wholeWordsOnly = new CheckMenuItem (MonoDevelop.Core.GettextCatalog.GetString ("_Whole words only"));
+			CheckMenuItem wholeWordsOnly = new CheckMenuItem (GettextCatalog.GetString ("_Whole words only"));
 			wholeWordsOnly.Active = IsWholeWordOnly;
 			wholeWordsOnly.DrawAsRadio = false;
 			wholeWordsOnly.Toggled += delegate {
@@ -293,7 +360,7 @@ namespace MonoDevelop.SourceEditor
 			searchEntry.Menu.Add (wholeWordsOnly);
 				
 			
-			CheckMenuItem regexSearch = new CheckMenuItem (MonoDevelop.Core.GettextCatalog.GetString ("_Regex search"));
+			CheckMenuItem regexSearch = new CheckMenuItem (GettextCatalog.GetString ("_Regex search"));
 			regexSearch.Active = SearchEngine == RegexSearchEngine;
 			regexSearch.DrawAsRadio = false;
 			regexSearch.Toggled += delegate {
@@ -301,10 +368,20 @@ namespace MonoDevelop.SourceEditor
 				UpdateSearchEntry ();
 			};
 			searchEntry.Menu.Add (regexSearch);
+			
+			CheckMenuItem inselectionSearch = new CheckMenuItem (GettextCatalog.GetString ("_Search In Selection"));
+			inselectionSearch.Active = IsInSelectionSearchMode;
+			inselectionSearch.DrawAsRadio = false;
+			inselectionSearch.Toggled += delegate {
+				IsInSelectionSearchMode = inselectionSearch.Active;
+				UpdateSearchEntry ();
+			};
+			searchEntry.Menu.Add (inselectionSearch);
+
 			List<string> history = GetHistory (seachHistoryProperty);
 			if (history.Count > 0) {
 				searchEntry.Menu.Add (new SeparatorMenuItem ());
-				MenuItem recentSearches = new MenuItem (MonoDevelop.Core.GettextCatalog.GetString ("Recent Searches"));
+				MenuItem recentSearches = new MenuItem (GettextCatalog.GetString ("Recent Searches"));
 				recentSearches.Sensitive = false;
 				searchEntry.Menu.Add (recentSearches);
 				
@@ -322,7 +399,7 @@ namespace MonoDevelop.SourceEditor
 					searchEntry.Menu.Add (recentItem);
 				}
 				searchEntry.Menu.Add (new SeparatorMenuItem ());
-				MenuItem clearRecentSearches = new MenuItem (MonoDevelop.Core.GettextCatalog.GetString ("Clear Recent Searches"));
+				MenuItem clearRecentSearches = new MenuItem (GettextCatalog.GetString ("Clear Recent Searches"));
 				clearRecentSearches.Activated += delegate {
 					StoreHistory (seachHistoryProperty, null);
 				};
@@ -362,9 +439,9 @@ namespace MonoDevelop.SourceEditor
 		
 		public void UpdateSearchPattern ()
 		{
-			searchEntry.Entry.Text = widget.TextEditor.SearchPattern;
-			widget.SetSearchPattern (widget.TextEditor.SearchPattern);
-			searchPattern = widget.TextEditor.SearchPattern;
+			searchEntry.Entry.Text = textEditor.SearchPattern;
+			SetSearchPattern (textEditor.SearchPattern);
+			searchPattern = textEditor.SearchPattern;
 //			UpdateSearchEntry ();
 		}
 		int curSearchResult = -1;
@@ -423,7 +500,7 @@ But I leave it in in the case I've missed something. Mike
 					buttonSearchBackward.Click ();
 					break;
 				case Gdk.Key.Escape:
-					widget.RemoveSearchWidget ();
+					RemoveSearchWidget ();
 					break;
 				case Gdk.Key.slash:
 					searchEntry.GrabFocus ();
@@ -466,7 +543,7 @@ But I leave it in in the case I've missed something. Mike
 						} else if (o == buttonSearchBackward) {
 							buttonSearchForward.GrabFocus ();
 						} else if (o == buttonSearchForward) {
-//							widget.TextEditor.GrabFocus ();
+//							textEditor.GrabFocus ();
 							searchEntry.Entry.GrabFocus ();
 						} else {
 							entryReplace.GrabFocus ();
@@ -477,7 +554,7 @@ But I leave it in in the case I've missed something. Mike
 							buttonSearchForward.GrabFocus ();
 						} else if (o == buttonSearchForward) {
 							searchEntry.Entry.GrabFocus ();
-//							widget.TextEditor.GrabFocus ();
+//							textEditor.GrabFocus ();
 						} else {
 							buttonSearchBackward.GrabFocus ();
 						}
@@ -493,41 +570,40 @@ But I leave it in in the case I've missed something. Mike
 		public bool IsReplaceMode {
 			get { return isReplaceMode; }
 			set {
-				if (value == isReplaceMode)
-					return;
-				
 				isReplaceMode = value;
 				searchButtonModeArrow.ArrowType = isReplaceMode ? ArrowType.Up : ArrowType.Down;
 				table.RowSpacing = isReplaceMode ? 6u : 0u;
 				foreach (Widget widget in replaceWidgets) {
 					widget.Visible = isReplaceMode;
 				}
+				if (textEditor.Document.ReadOnly)
+					buttonSearchMode.Visible = false;
 			}
 		}
 		protected override void OnFocusChildSet (Widget widget)
 		{
 			base.OnFocusChildSet (widget);
-			ISegment mainResult = this.widget.TextEditor.TextViewMargin.MainSearchResult;
-			this.widget.TextEditor.TextViewMargin.HideSelection = widget == table && mainResult != null &&
-				this.widget.TextEditor.IsSomethingSelected && this.widget.TextEditor.SelectionRange.Offset == mainResult.Offset && this.widget.TextEditor.SelectionRange.EndOffset == mainResult.EndOffset;
+			var mainResult = textEditor.TextViewMargin.MainSearchResult;
+			textEditor.TextViewMargin.HideSelection = widget == table && !mainResult.IsInvalid &&
+				textEditor.IsSomethingSelected && textEditor.SelectionRange.Offset == mainResult.Offset && textEditor.SelectionRange.EndOffset == mainResult.EndOffset;
 			
-			if (this.widget.TextEditor.TextViewMargin.HideSelection)
-				this.widget.TextEditor.QueueDraw ();
+			if (textEditor.TextViewMargin.HideSelection)
+				textEditor.QueueDraw ();
 		}
 		
 		protected override void OnDestroyed ()
 		{
-			this.widget.TextEditor.TextViewMargin.HideSelection = false;
-			widget.TextEditor.Caret.PositionChanged -= HandleWidgetTextEditorCaretPositionChanged;
-			widget.TextEditor.TextViewMargin.SearchRegionsUpdated -= HandleWidgetTextEditorTextViewMarginSearchRegionsUpdated;
-			widget.TextEditorContainer.SizeAllocated -= HandleViewTextEditorhandleSizeAllocated;
+			textEditor.TextViewMargin.HideSelection = false;
+			textEditor.Caret.PositionChanged -= HandleWidgetTextEditorCaretPositionChanged;
+			textEditor.TextViewMargin.SearchRegionsUpdated -= HandleWidgetTextEditorTextViewMarginSearchRegionsUpdated;
+			SizeAllocated -= HandleViewTextEditorhandleSizeAllocated;
+			textEditorContainer.SizeAllocated -= HandleViewTextEditorhandleSizeAllocated;
 			
 			// SearchPatternChanged -= UpdateSearchPattern;
 			ReplacePatternChanged -= UpdateReplacePattern;
 			
-			if (widget != null) {
-				widget.TextEditor.QueueDraw ();
-				widget = null;
+			if (frame != null) {
+				textEditor.QueueDraw ();
 			}
 			base.OnDestroyed ();
 		}
@@ -542,23 +618,23 @@ But I leave it in in the case I've missed something. Mike
 		
 		void StoreWidgetState ()
 		{
-			// vSave  = widget.TextEditor.VAdjustment.Value;
-			// hSave  = widget.TextEditor.HAdjustment.Value;
-			caretSave =  widget.TextEditor.Caret.Location;
+			// vSave  = textEditor.VAdjustment.Value;
+			// hSave  = textEditor.HAdjustment.Value;
+			caretSave =  textEditor.Caret.Location;
 		}
 		
 		void GotoResult (SearchResult result)
 		{
 			try {
 				if (result == null) {
-					widget.TextEditor.ClearSelection ();
+					textEditor.ClearSelection ();
 					return;
 				}
-				widget.TextEditor.StopSearchResultAnimation ();
-				widget.TextEditor.Caret.Offset = result.EndOffset;
-				widget.TextEditor.SetSelection (result.Offset, result.EndOffset);
-				widget.TextEditor.CenterToCaret ();
-				widget.TextEditor.AnimateSearchResult (result);
+				textEditor.StopSearchResultAnimation ();
+				textEditor.Caret.Location = textEditor.OffsetToLocation (result.EndOffset);
+				textEditor.SetSelection (result.Offset, result.EndOffset);
+				textEditor.CenterToCaret ();
+				textEditor.AnimateSearchResult (result);
 			} catch (System.Exception) {
 			}
 		}
@@ -619,20 +695,20 @@ But I leave it in in the case I've missed something. Mike
 		void SetIsCaseSensitive (bool value)
 		{
 			IsCaseSensitive = value;
-			widget.SetSearchOptions ();
+			SetSearchOptions ();
 		}
 		
 		void SetIsWholeWordOnly (bool value)
 		{
 			PropertyService.Set ("IsWholeWordOnly", value);
-			widget.SetSearchOptions ();
+			SetSearchOptions ();
 		}
 		
 		void SetIsRegexSearch (bool value)
 		{
 			PropertyService.Set ("BufferSearchEngine", value ? RegexSearchEngine : 
 			                                                   DefaultSearchEngine);
-			widget.SetSearchOptions ();
+			SetSearchOptions ();
 		}
 		
 //		void UpdateSearchPattern (object sender, EventArgs args)
@@ -646,8 +722,8 @@ But I leave it in in the case I've missed something. Mike
 		{
 			if (oldPattern != SearchPattern) {
 				oldPattern = SearchPattern;
-				widget.SetSearchOptions ();
-				result = widget.TextEditor.SearchForward (widget.TextEditor.Document.LocationToOffset (caretSave));
+				SetSearchOptions ();
+				result = textEditor.SearchForward (textEditor.Document.LocationToOffset (caretSave));
 			}
 			
 			GotoResult (result);
@@ -663,10 +739,10 @@ But I leave it in in the case I've missed something. Mike
 				return;
 			}
 			
-		//	bool error = result == null && !String.IsNullOrEmpty (SearchPattern);
+			//	bool error = result == null && !String.IsNullOrEmpty (SearchPattern);
 			string errorMsg;
-			bool valid = widget.TextEditor.SearchEngine.IsValidPattern (searchPattern, out errorMsg);
-		//	error |= !valid;
+			bool valid = textEditor.SearchEngine.IsValidPattern (searchPattern, out errorMsg);
+			//	error |= !valid;
 			
 			if (!valid) {
 				IdeApp.Workbench.StatusBar.ShowError (errorMsg);
@@ -674,7 +750,7 @@ But I leave it in in the case I've missed something. Mike
 				IdeApp.Workbench.StatusBar.ShowReady ();
 			}
 			
-			if (!valid || widget.TextEditor.TextViewMargin.SearchResultMatchCount == 0) {
+			if (!valid || textEditor.TextViewMargin.SearchResultMatchCount == 0) {
 				//resultInformLabel.Markup = "<span foreground=\"#000000\" background=\"" + MonoDevelop.Components.PangoCairoHelper.GetColorString (GotoLineNumberWidget.errorColor) + "\">" + GettextCatalog.GetString ("Not found") + "</span>";
 				resultInformLabel.Text = GettextCatalog.GetString ("Not found");
 				resultInformLabelEventBox.ModifyBg (StateType.Normal, GotoLineNumberWidget.errorColor);
@@ -682,9 +758,9 @@ But I leave it in in the case I've missed something. Mike
 			} else {
 				int resultIndex = 0;
 				int foundIndex = -1;
-				int caretOffset = widget.TextEditor.Caret.Offset;
-				ISegment foundSegment = null;
-				foreach (ISegment searchResult in widget.TextEditor.TextViewMargin.SearchResults) {
+				int caretOffset = textEditor.Caret.Offset;
+				TextSegment foundSegment = TextSegment.Invalid;
+				foreach (var searchResult in textEditor.TextViewMargin.SearchResults) {
 					if (searchResult.Offset <= caretOffset && caretOffset <= searchResult.EndOffset) {
 						foundIndex = resultIndex + 1;
 						foundSegment = searchResult;
@@ -693,14 +769,14 @@ But I leave it in in the case I've missed something. Mike
 					resultIndex++;
 				}
 				if (foundIndex != -1) {
-					resultInformLabel.Text = String.Format (GettextCatalog.GetString ("{0} of {1}"), foundIndex, widget.TextEditor.TextViewMargin.SearchResultMatchCount);
+					resultInformLabel.Text = String.Format (GettextCatalog.GetString ("{0} of {1}"), foundIndex, textEditor.TextViewMargin.SearchResultMatchCount);
 				} else {
-					resultInformLabel.Text = String.Format (GettextCatalog.GetPluralString ("{0} match", "{0} matches", widget.TextEditor.TextViewMargin.SearchResultMatchCount), widget.TextEditor.TextViewMargin.SearchResultMatchCount);
+					resultInformLabel.Text = String.Format (GettextCatalog.GetPluralString ("{0} match", "{0} matches", textEditor.TextViewMargin.SearchResultMatchCount), textEditor.TextViewMargin.SearchResultMatchCount);
 				}
 				resultInformLabelEventBox.ModifyBg (StateType.Normal, searchEntry.Entry.Style.Base (searchEntry.Entry.State));
 				resultInformLabel.ModifyFg (StateType.Normal, searchEntry.Entry.Style.Foreground (StateType.Insensitive));
-				widget.TextEditor.TextViewMargin.HideSelection = FocusChild == this.table;
-				widget.TextEditor.TextViewMargin.MainSearchResult = foundSegment;
+				textEditor.TextViewMargin.HideSelection = FocusChild == table;
+				textEditor.TextViewMargin.MainSearchResult = foundSegment;
 			}
 		} 
 		
@@ -714,7 +790,80 @@ But I leave it in in the case I've missed something. Mike
 		{
 			entryReplace.Text = replacePattern;
 		}
+
+		internal void SetSearchPattern ()
+		{
+			string selectedText = SourceEditorWidget.FormatPatternToSelectionOption (textEditor.SelectedText);
+			
+			if (!String.IsNullOrEmpty (selectedText)) {
+				SetSearchPattern (selectedText);
+				SearchAndReplaceWidget.searchPattern = selectedText;
+				SearchAndReplaceWidget.UpdateSearchHistory (selectedText);
+				textEditor.TextViewMargin.MainSearchResult = textEditor.SelectionRange;
+			}
+		}
 		
+		public void SetSearchPattern (string searchPattern)
+		{
+			textEditor.SearchPattern = searchPattern;
+		}
+		
+
+		public static SearchResult FindNext (TextEditor textEditor)
+		{
+			SearchResult result = textEditor.FindNext (true);
+			textEditor.CenterToCaret ();
+
+			if (result == null) {
+				IdeApp.Workbench.StatusBar.ShowError (GettextCatalog.GetString ("Search pattern not found"));
+			} else if (result.SearchWrapped) {
+				IdeApp.Workbench.StatusBar.ShowMessage (
+					new Image (Stock.Find, IconSize.Menu),
+					GettextCatalog.GetString ("Reached bottom, continued from top")
+				);
+			} else {
+				IdeApp.Workbench.StatusBar.ShowReady ();
+			}
+			return result;
+		}
+
+		public static SearchResult FindPrevious (TextEditor textEditor)
+		{
+			SearchResult result = textEditor.FindPrevious (true);
+
+			textEditor.CenterToCaret ();
+			if (result == null) {
+				IdeApp.Workbench.StatusBar.ShowError (GettextCatalog.GetString ("Search pattern not found"));
+			} else if (result.SearchWrapped) {
+				IdeApp.Workbench.StatusBar.ShowMessage (
+					new Image (Stock.Find, IconSize.Menu),
+					GettextCatalog.GetString ("Reached top, continued from bottom")
+				);
+			} else {
+				IdeApp.Workbench.StatusBar.ShowReady ();
+			}
+			return result;
+		}
+
+		public void Replace ()
+		{
+			textEditor.Replace (ReplacePattern);
+			textEditor.GrabFocus ();
+		}
+		
+		public void ReplaceAll ()
+		{
+			int number = textEditor.ReplaceAll (ReplacePattern);
+			if (number == 0) {
+				IdeApp.Workbench.StatusBar.ShowError (GettextCatalog.GetString ("Search pattern not found"));
+			} else {
+				IdeApp.Workbench.StatusBar.ShowMessage (
+					GettextCatalog.GetPluralString ("Found and replaced one occurrence",
+					                                "Found and replaced {0} occurrences", number, number));
+			}
+			textEditor.GrabFocus ();
+		}
+
 		internal static bool inReplaceUpdate = false;
 		internal static void FireReplacePatternChanged ()
 		{
@@ -726,5 +875,10 @@ But I leave it in in the case I've missed something. Mike
 		
 		internal static event EventHandler ReplacePatternChanged;
 
+		void RemoveSearchWidget ()
+		{
+			textEditor.HighlightSearchPattern = false;
+			Destroy ();
+		}
 	}
 }
