@@ -9,9 +9,23 @@
 //   (C) 2007 Matt Ward
 //   (C) 2008 Novell, Inc (http://www.novell.com)
 // 
-// License:
-//   Derived from LGPL files (Matt Ward)
-//   All code since then is MIT/X11
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+// 
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
 
 using System;
 using System.Linq;
@@ -21,16 +35,17 @@ using System.Xml.Schema;
 
 using MonoDevelop.Core;
 using MonoDevelop.Components.Commands;
-using MonoDevelop.Projects.Gui;
-using MonoDevelop.Projects.Gui.Completion;
+using MonoDevelop.Ide.CodeCompletion;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.XmlEditor.Completion;
 using MonoDevelop.Xml.StateEngine;
+using MonoDevelop.Ide.Tasks;
+using MonoDevelop.Ide;
+using MonoDevelop.Ide.CodeFormatting;
+using Mono.TextEditor;
 
 namespace MonoDevelop.XmlEditor
 {
-	
-	
 	public class XmlTextEditorExtension : MonoDevelop.XmlEditor.Gui.BaseXmlEditorExtension
 	{
 		const string TextXmlMimeType = "text/xml";
@@ -54,24 +69,17 @@ namespace MonoDevelop.XmlEditor
 			return IsFileNameHandled (doc.Name);
 		}
 		
-		protected override IEnumerable<string> SupportedExtensions {
-			get { throw new System.InvalidOperationException (); }
-		}
- 
-		
 		public override void Initialize ()
 		{
 			base.Initialize ();
-			XmlEditorAddInOptions.PropertyChanged += XmlEditorPropertyChanged;
+			XmlEditorOptions.XmlFileAssociationChanged += HandleXmlFileAssociationChanged;
 			XmlSchemaManager.UserSchemaAdded += UserSchemaAdded;
 			XmlSchemaManager.UserSchemaRemoved += UserSchemaRemoved;
-			SetInitialValues();
+			SetDefaultSchema ();
 			
-			MonoDevelop.SourceEditor.SourceEditorView view = 
-				Document.GetContent<MonoDevelop.SourceEditor.SourceEditorView> ();
-			if (view != null && view.Document.SyntaxMode == null) {
-				Mono.TextEditor.Highlighting.SyntaxMode mode = 
-					Mono.TextEditor.Highlighting.SyntaxModeService.GetSyntaxMode (ApplicationXmlMimeType);
+			var view = Document.GetContent<MonoDevelop.SourceEditor.SourceEditorView> ();
+			if (view != null && string.IsNullOrEmpty (view.Document.MimeType)) {
+				var mode = Mono.TextEditor.Highlighting.SyntaxModeService.GetSyntaxMode (view.Document, ApplicationXmlMimeType);
 				if (mode != null)
 					view.Document.SyntaxMode = mode;
 				else
@@ -79,13 +87,20 @@ namespace MonoDevelop.XmlEditor
 					    + ApplicationXmlMimeType + "'.");
 			}
 		}
+
+		void HandleXmlFileAssociationChanged (object sender, XmlFileAssociationChangedEventArgs e)
+		{
+			var filename = document.FileName;
+			if (filename != null && filename.ToString ().EndsWith (e.Extension))
+				SetDefaultSchema ();
+		}
 		
 		bool disposed;
 		public override void Dispose()
 		{
 			if (!disposed) {
 				disposed = false;
-				XmlEditorAddInOptions.PropertyChanged -= XmlEditorPropertyChanged;
+				XmlEditorOptions.XmlFileAssociationChanged -= HandleXmlFileAssociationChanged;
 				XmlSchemaManager.UserSchemaAdded -= UserSchemaAdded;
 				XmlSchemaManager.UserSchemaRemoved -= UserSchemaRemoved;
 				base.Dispose ();
@@ -94,38 +109,33 @@ namespace MonoDevelop.XmlEditor
 		
 		#region Code completion
 		
-//		IEditableTextBuffer GetBuffer ()
-//		{
-//			IEditableTextBuffer buf = Document.GetContent<IEditableTextBuffer> ();
-//			System.Diagnostics.Debug.Assert (buf != null);
-//			return buf;
-//		}
-		
-		XmlElementPath GetCurrentPath ()
+		XmlElementPath GetElementPath ()
 		{
-			return ConvertPath (Tracker.Engine.Nodes.OfType<XElement> ().Reverse ());
+			return ConvertPath (GetCurrentPath ());
 		}
 		
-		static XmlElementPath ConvertPath (IEnumerable<XElement> path)
+		XmlElementPath ConvertPath (IList<XObject> path)
 		{
-			XmlElementPath elementPath = new XmlElementPath ();
-			var namespaces = new Dictionary<string, string> ();
-			string defaultNamespace = null;
-			foreach (XElement el in path) {
-				foreach (XAttribute att in el.Attributes) {
-					if (att.Name.HasPrefix) {
-						if (att.Name.Prefix == "xmlns")
-							namespaces [att.Name.Name] = att.Value;
-					} else {
-						if (att.Name.Name == "xmlns")
-							defaultNamespace = att.Value;
+			var elementPath = new XmlElementPath ();
+			
+			if (defaultSchemaCompletionData != null && !string.IsNullOrEmpty (defaultSchemaCompletionData.NamespaceUri))
+				elementPath.Namespaces.AddPrefix (defaultSchemaCompletionData.NamespaceUri, defaultNamespacePrefix ?? "");
+			
+			foreach (var obj in path) {
+				var el = obj as XElement;
+				if (el == null)
+					continue;
+				foreach (var att in el.Attributes) {
+					if (!string.IsNullOrEmpty (att.Value)) {
+						if (att.Name.HasPrefix) {
+							if (att.Name.Prefix == "xmlns")
+								elementPath.Namespaces.AddPrefix (att.Value, att.Name.Name);
+						} else if (att.Name.Name == "xmlns") {
+								elementPath.Namespaces.AddPrefix (att.Value, "");
+						}
 					}
 				}
-				string ns = null;
-				if (el.Name.HasPrefix)
-					namespaces.TryGetValue (el.Name.Prefix, out ns);
-				else
-					ns = defaultNamespace;
+				string ns = elementPath.Namespaces.GetNamespace (el.Name.HasPrefix? el.Name.Prefix : "");
 				QualifiedName qn = new QualifiedName (el.Name.Name, ns, el.Name.Prefix ?? String.Empty);
 				elementPath.Elements.Add (qn);
 			}
@@ -134,13 +144,13 @@ namespace MonoDevelop.XmlEditor
 		
 		protected override void GetElementCompletions (CompletionDataList list)
 		{	
-			XmlElementPath path = GetCurrentPath ();
+			var path = GetElementPath ();
 			if (path.Elements.Count > 0) {
 				IXmlCompletionProvider schema = FindSchema (path);
 				if (schema == null)
 					schema = inferredCompletionData;
 				if (schema != null) {
-					ICompletionData[] completionData = schema.GetChildElementCompletionData (path);
+					var completionData = schema.GetChildElementCompletionData (path);
 					if (completionData != null)
 						list.AddRange (completionData);
 				}
@@ -149,36 +159,32 @@ namespace MonoDevelop.XmlEditor
 			} else if (inferredCompletionData != null) {
 				list.AddRange (inferredCompletionData.GetElementCompletionData ());
 			}
+			AddMiscBeginTags (list);
 		}
 		
-		protected override void GetAttributeCompletions (CompletionDataList list, IAttributedXObject attributedOb, 
-		                                                 Dictionary<string, string> existingAtts)
+		protected override CompletionDataList GetAttributeCompletions (IAttributedXObject attributedOb,
+			Dictionary<string, string> existingAtts)
 		{
-			XmlElementPath path = GetCurrentPath ();
+			var path = GetElementPath ();
 			if (path.Elements.Count > 0) {
 				IXmlCompletionProvider schema = FindSchema (path);
 				if (schema == null)
 					schema = inferredCompletionData;
-				if (schema != null) {
-					ICompletionData[] completionData = schema.GetAttributeCompletionData (path);
-					if (completionData != null)
-						list.AddRange (completionData);
-				}
+				if (schema != null)
+					return schema.GetAttributeCompletionData (path);
 			}
+			return null;
 		}
 		
-		protected override void GetAttributeValueCompletions (CompletionDataList list, IAttributedXObject attributedOb,
-		                                                      XAttribute att)
+		protected override CompletionDataList GetAttributeValueCompletions (IAttributedXObject attributedOb, XAttribute att)
 		{
-			XmlElementPath path = GetCurrentPath ();
+			var path = GetElementPath ();
 			if (path.Elements.Count > 0) {
-				XmlSchemaCompletionData schema = FindSchema (path);
-				if (schema != null) {
-					ICompletionData[] completionData = schema.GetAttributeValueCompletionData (path, att.Name.FullName);
-					if (completionData != null)
-						list.AddRange (completionData);
-				}
+				var schema = FindSchema (path);
+				if (schema != null)
+					return schema.GetAttributeValueCompletionData (path, att.Name.FullName);
 			}
+			return null;
 		}
 		
 		#endregion
@@ -238,7 +244,7 @@ namespace MonoDevelop.XmlEditor
 		public XmlSchemaObject GetSchemaObjectSelected (XmlSchemaCompletionData currentSchemaCompletionData)
 		{
 			// Find element under cursor.
-			XmlElementPath path = GetCurrentPath ();
+			XmlElementPath path = GetElementPath ();
 			
 			//attribute name under cursor, if valid
 			string attributeName = null;
@@ -258,7 +264,7 @@ namespace MonoDevelop.XmlEditor
 				XmlSchemaElement element = schemaCompletionData.FindElement(path);
 				schemaObject = element;
 				if (element != null) {
-					if (attributeName.Length > 0) {
+					if (!string.IsNullOrEmpty (attributeName)) {
 						XmlSchemaAttribute attribute = schemaCompletionData.FindAttribute(element, attributeName);
 						if (attribute != null) {
 							if (currentSchemaCompletionData != null) {
@@ -380,57 +386,30 @@ namespace MonoDevelop.XmlEditor
 		
 		#region Settings handling
 		
-		void SetDefaultSchema (string fileName)
+		void SetDefaultSchema ()
 		{
-			if (fileName == null) {
+			var filename = document.FileName;
+			if (filename == null)
 				return;
-			}
-			string extension = System.IO.Path.GetExtension (fileName).ToLower ();
-			defaultSchemaCompletionData = XmlSchemaManager.GetSchemaCompletionData (extension);
+			
+			defaultSchemaCompletionData = XmlSchemaManager.GetSchemaCompletionDataForFileName (filename);
 			if (defaultSchemaCompletionData != null)
 				inferredCompletionData = null;
 			else
 				QueueInference ();
-			defaultNamespacePrefix = XmlSchemaManager.GetNamespacePrefix (extension);
+			defaultNamespacePrefix = XmlSchemaManager.GetNamespacePrefixForFileName (filename);
 		}
 		
 		/// Updates the default schema association since the schema may have been added.
 		void UserSchemaAdded (object source, EventArgs e)
 		{	
-			SetDefaultSchema (Document.Name);
+			SetDefaultSchema ();
 		}
 		
 		// Updates the default schema association since the schema may have been removed.
 		void UserSchemaRemoved (object source, EventArgs e)
 		{
-			SetDefaultSchema (Document.Name);
-		}
-		
-		void XmlEditorPropertyChanged (object sender, PropertyChangedEventArgs args)
- 		{
-			switch (args.Key) {
-			case "AutoCompleteElements":
-				AutoCompleteClosingTags = XmlEditorAddInOptions.AutoCompleteElements;
-				break;
-			case "ShowSchemaAnnotation":
-//				showSchemaAnnotation = XmlEditorAddInOptions.ShowSchemaAnnotation;
-				break;
-			default:
-				string extension = System.IO.Path.GetExtension (Document.Name).ToLower ();
-				if (args.Key == extension) {
-					SetDefaultSchema (Document.Name);
-				} else {
-					LoggingService.LogError ("Unhandled property change in XmlTextEditorExtension: " + args.Key);
-				}
-				break;
-			}
-		}
-		
-		void SetInitialValues()
-		{
-//			showSchemaAnnotation = XmlEditorAddInOptions.ShowSchemaAnnotation;
-			AutoCompleteClosingTags = XmlEditorAddInOptions.AutoCompleteElements;
-			SetDefaultSchema (Document.Name);
+			SetDefaultSchema ();
 		}
 		
 		#endregion
@@ -466,20 +445,22 @@ namespace MonoDevelop.XmlEditor
 		{			
 			if (fileName == null)
 				return false;
-			
 			if (System.IO.Path.IsPathRooted (fileName)) {
 				string vfsname = fileName.Replace ("%", "%25").Replace ("#", "%23").Replace ("?", "%3F");
-				string mimeType = MonoDevelop.Core.Gui.Services.PlatformService.GetMimeTypeForUri (vfsname);
+				string mimeType = DesktopService.GetMimeTypeForUri (vfsname);
 				if (IsMimeTypeHandled (mimeType))
 					return true;
 			}
 			
-			return XmlFileExtensions.IsXmlFileExtension (System.IO.Path.GetExtension (fileName));
+			return XmlFileAssociationManager.IsXmlFileName (fileName);
 		}
 		
 		public static bool IsMimeTypeHandled (string mimeType)
 		{
-			return (mimeType != null && (mimeType == TextXmlMimeType || mimeType == ApplicationXmlMimeType));
+			foreach (var m in DesktopService.GetMimeTypeInheritanceChain (mimeType))
+				if (m == TextXmlMimeType || m == ApplicationXmlMimeType)
+					return true;
+			return false;
 		}
 			
 		#endregion
@@ -490,9 +471,9 @@ namespace MonoDevelop.XmlEditor
 		{
 			bool result;
 			
-			if (TextEditorProperties.IndentStyle == IndentStyle.Smart && key == Gdk.Key.Return) {
+			if (Document.Editor.Options.IndentStyle == IndentStyle.Smart && key == Gdk.Key.Return) {
 				result = base.KeyPress (key, keyChar, modifier);
-				SmartIndentLine (Editor.CursorLine);
+				SmartIndentLine (Editor.Caret.Line);
 				return result;
 			}
 			return base.KeyPress (key, keyChar, modifier);
@@ -529,7 +510,7 @@ namespace MonoDevelop.XmlEditor
 			while (i > 0) {
 				char c = Editor.GetCharAt (i);
 				if (c == '\n' || c == '\r')
-					return Editor.GetText (i + 1, indentEnd);
+					return Editor.GetTextBetween (i + 1, indentEnd);
 				if (!char.IsWhiteSpace (c))
 					indentEnd--;
 				i--;
@@ -553,58 +534,11 @@ namespace MonoDevelop.XmlEditor
 			//FIXME: implement
 		}
 		
-		[CommandHandler (Commands.Format)]
-		public void FormatCommand ()
-		{
-			MonoDevelop.Ide.Gui.IdeApp.Services.TaskService.ClearExceptCommentTasks ();
-			
-			using (IProgressMonitor monitor = XmlEditorService.GetMonitor ()) {
-				bool selection = (Editor.SelectionEndPosition - Editor.SelectionStartPosition) > 0;
-				string xml = selection? Editor.SelectedText : Editor.Text;
-				XmlDocument doc = XmlEditorService.ValidateWellFormedness (monitor, xml, FileName);
-				if (doc == null)
-					return;
-				
-				//if there's a line indent at the current location, prepend that to all new lines
-				string extraIndent = null;
-				if (selection)
-					extraIndent = GetPositionIndent (Editor.SelectionStartPosition);
-				
-				string formattedXml = XmlEditorService.IndentedFormat (xml);
-				
-				//convert newlines and prepend extra indents to each line if needed
-				bool nonNativeNewline = (Editor.NewLine != Environment.NewLine);
-				bool hasExtraIndent = !string.IsNullOrEmpty (extraIndent);
-				if (hasExtraIndent || nonNativeNewline) {
-					System.Text.StringBuilder builder = new System.Text.StringBuilder (formattedXml);
-					
-					if (nonNativeNewline)
-						builder.Replace (Environment.NewLine, Editor.NewLine);
-					
-					if (hasExtraIndent) {
-						builder.Replace (Editor.NewLine, Editor.NewLine + extraIndent);
-						if (formattedXml.EndsWith (Environment.NewLine))
-							builder.Remove (builder.Length - 1 - extraIndent.Length, extraIndent.Length);
-					}
-					formattedXml = builder.ToString ();
-				}
-				
-				Editor.BeginAtomicUndo ();
-				if (selection) {
-					Editor.SelectedText = formattedXml;
-				} else {
-					Editor.DeleteText (0, Editor.TextLength);
-					Editor.InsertText (0, formattedXml);
-				}
-				Editor.EndAtomicUndo ();
-			}
-		}
-		
 		[CommandHandler (Commands.CreateSchema)]
 		public void CreateSchemaCommand ()
 		{
 			try {
-				MonoDevelop.Ide.Gui.IdeApp.Services.TaskService.ClearExceptCommentTasks ();
+				TaskService.Errors.Clear ();
 				string xml = Editor.Text;
 				using (IProgressMonitor monitor = XmlEditorService.GetMonitor ()) {
 					XmlDocument doc = XmlEditorService.ValidateWellFormedness (monitor, xml, FileName);
@@ -612,9 +546,9 @@ namespace MonoDevelop.XmlEditor
 						return;
 					monitor.BeginTask (GettextCatalog.GetString ("Creating schema..."), 0);
 					try {
-						string schema = XmlEditorService.CreateSchema (xml);
+						string schema = XmlEditorService.CreateSchema (Document, xml);
 						string fileName = XmlEditorService.GenerateFileName (FileName, "{0}.xsd");
-						MonoDevelop.Ide.Gui.IdeApp.Workbench.NewDocument (fileName, "application/xml", schema);
+						IdeApp.Workbench.NewDocument (fileName, "application/xml", schema);
 						monitor.ReportSuccess (GettextCatalog.GetString ("Schema created."));
 					} catch (Exception ex) {
 						string msg = GettextCatalog.GetString ("Error creating XML schema.");
@@ -623,7 +557,7 @@ namespace MonoDevelop.XmlEditor
 					}
 				}
 			} catch (Exception ex) {
-				MonoDevelop.Core.Gui.MessageService.ShowError(ex.Message);
+				MessageService.ShowError (ex.Message);
 			}
 		}
 		
@@ -632,10 +566,10 @@ namespace MonoDevelop.XmlEditor
 		{
 			if (!string.IsNullOrEmpty (stylesheetFileName)) {
 				try {
-					MonoDevelop.Ide.Gui.IdeApp.Workbench.OpenDocument (stylesheetFileName);
+					IdeApp.Workbench.OpenDocument (stylesheetFileName);
 				} catch (Exception ex) {
 					MonoDevelop.Core.LoggingService.LogError ("Could not open document.", ex);
-					MonoDevelop.Core.Gui.MessageService.ShowException (ex, "Could not open document.");
+					MessageService.ShowException (ex, "Could not open document.");
 				}
 			}
 		}
@@ -657,21 +591,21 @@ namespace MonoDevelop.XmlEditor
 				// Open schema if resolved
 				if (schemaObject != null && schemaObject.SourceUri != null && schemaObject.SourceUri.Length > 0) {
 					string schemaFileName = schemaObject.SourceUri.Replace ("file:/", String.Empty);
-					MonoDevelop.Ide.Gui.IdeApp.Workbench.OpenDocument (
+					IdeApp.Workbench.OpenDocument (
 					    schemaFileName,
 					    Math.Max (1, schemaObject.LineNumber),
-					    Math.Max (1, schemaObject.LinePosition), true);
+					    Math.Max (1, schemaObject.LinePosition));
 				}
 			} catch (Exception ex) {
 				MonoDevelop.Core.LoggingService.LogError ("Could not open document.", ex);
-				MonoDevelop.Core.Gui.MessageService.ShowException (ex, "Could not open document.");
+				MessageService.ShowException (ex, "Could not open document.");
 			}
 		}
 		
 		[CommandHandler (Commands.Validate)]
 		public void ValidateCommand ()
 		{
-			MonoDevelop.Ide.Gui.IdeApp.Services.TaskService.ClearExceptCommentTasks ();
+			TaskService.Errors.Clear ();
 			using (IProgressMonitor monitor = XmlEditorService.GetMonitor()) {
 				if (IsSchema)
 					XmlEditorService.ValidateSchema (monitor, Editor.Text, FileName);
@@ -720,9 +654,9 @@ namespace MonoDevelop.XmlEditor
 					string newFileName = XmlEditorService.GenerateFileName (FileName, "-transformed{0}.xml");
 					
 					monitor.BeginTask (GettextCatalog.GetString ("Executing transform..."), 1);
-					using (XmlTextWriter output = XmlEditorService.CreateXmlTextWriter()) {
+					using (XmlTextWriter output = XmlEditorService.CreateXmlTextWriter(Document)) {
 						xslt.Transform (doc, null, output);
-						MonoDevelop.Ide.Gui.IdeApp.Workbench.NewDocument (
+						IdeApp.Workbench.NewDocument (
 						    newFileName, "application/xml", output.ToString ());
 					}
 					monitor.ReportSuccess (GettextCatalog.GetString ("Transform completed."));
@@ -735,11 +669,48 @@ namespace MonoDevelop.XmlEditor
 			}
 		}
 		
+		/*
+		[CommandUpdateHandler (CodeFormattingCommands.FormatSelection)]
+		internal void UpdateFormatSelection (CommandInfo info)
+		{
+			info.Enabled = false;
+		}
+		
+		[CommandHandler (CodeFormattingCommands.FormatSelection)]
+		internal void FormatSelection (CommandInfo info)
+		{
+			throw new NotImplementedException ();
+		}
+		*/
+		
+		[CommandUpdateHandler (CodeFormattingCommands.FormatBuffer)]
+		internal void UpdateFormatDocument (CommandInfo info)
+		{
+			//we know there is an XML formatter because this addin registers it
+			info.Enabled = true;
+		}
+		
+		//we have to implement the command here simply to force the document mimetype
+		//FIXME: instead we should register the XML mimetype additions to the desktopservice
+		[CommandHandler (CodeFormattingCommands.FormatBuffer)]
+		internal void FormatDocument ()
+		{
+			var formatter = CodeFormatterService.GetFormatter (TextXmlMimeType);
+			using (var undo = Editor.OpenUndoGroup ()) {
+				var loc = Editor.Caret.Location;
+				var text = formatter.FormatText (Document.Project != null ? Document.Project.Policies : null, Editor.Text);
+				if (text != null) {
+					Editor.Replace (0, Editor.Length, text);
+					Editor.Caret.Location = loc;
+				}
+			}
+		}
+		
 		string GetFileContent (string fileName)
 		{
 			MonoDevelop.Projects.Text.IEditableTextFile tf =
-				MonoDevelop.DesignerSupport.OpenDocumentFileProvider.Instance.GetEditableTextFile (fileName);
-			if (tf != null)
+				MonoDevelop.Ide.TextFileProvider.Instance.GetEditableTextFile (fileName);
+ 			if (tf != null)
 				return tf.Text;
 			System.IO.StreamReader reader = new System.IO.StreamReader (fileName, true);
 			return reader.ReadToEnd();
@@ -753,14 +724,14 @@ namespace MonoDevelop.XmlEditor
 			if (defaultSchemaCompletionData != null || doc == null || doc.XDocument == null || inferenceQueued)
 				return;
 			if (inferredCompletionData == null
-			    || (doc.ParseTime - inferredCompletionData.TimeStamp).TotalSeconds >= 5
+			    || (doc.LastWriteTimeUtc - inferredCompletionData.TimeStampUtc).TotalSeconds >= 5
 			        && doc.Errors.Count <= inferredCompletionData.ErrorCount)
 			{
 				inferenceQueued = true;
 				System.Threading.ThreadPool.QueueUserWorkItem (delegate {
 					InferredXmlCompletionProvider newData = new InferredXmlCompletionProvider ();
 					newData.Populate (doc.XDocument);
-					newData.TimeStamp = DateTime.Now;
+					newData.TimeStampUtc = DateTime.UtcNow;
 					newData.ErrorCount = doc.Errors.Count;
 					this.inferenceQueued = false;
 					this.inferredCompletionData = newData;
@@ -774,4 +745,4 @@ namespace MonoDevelop.XmlEditor
 			base.OnParsedDocumentUpdated ();
 		}		
 	}
-}
+}
\ No newline at end of file
