@@ -1,4 +1,5 @@
 using System.Collections.Generic;
+using System.Linq;
 
 using MonoDevelop.Projects;
 using MonoDevelop.Core;
@@ -11,12 +12,12 @@ namespace MonoDevelop.VersionControl
 	{
 		public static bool Publish (IWorkspaceObject entry, FilePath localPath, bool test)
 		{
-			if (test)
-				return true;
-
 			if (!VersionControlService.CheckVersionControlInstalled ())
 				return false;
 
+			if (test)
+				return VersionControlService.GetRepository (entry) == null;
+
 			List<FilePath> files = new List<FilePath> ();
 
 			// Build the list of files to be checked in			
@@ -56,8 +57,9 @@ namespace MonoDevelop.VersionControl
 
 		static void GetFiles (List<FilePath> files, IWorkspaceObject entry)
 		{
+			// Ensure that we strip out all linked files from outside of the solution/projects path.
 			if (entry is IWorkspaceFileObject)
-				files.AddRange (((IWorkspaceFileObject)entry).GetItemFiles (true));
+				files.AddRange (((IWorkspaceFileObject)entry).GetItemFiles (true).Where (file => file.IsChildPathOf (entry.BaseDirectory)));
 		}
 		
 		public static bool CanPublish (Repository vc, string path, bool isDir) {
