@@ -11,12 +11,24 @@ namespace Mono.TextEditor.Highlighting
 		}
 		string[] patterns;
 		
+		Regex ()
+		{
+		}
+		
 		public Regex (string pattern)
 		{
 			this.Pattern = pattern;
 			this.patterns = pattern.Split ('|');
 		}
 		
+		public Regex Clone ()
+		{
+			var newRegex = new Regex ();
+			newRegex.Pattern = Pattern;
+			newRegex.patterns = patterns;
+			return newRegex;
+		}
+		
 		public RegexMatch TryMatch (string doc, int offset)
 		{
 			foreach (string pattern in patterns) {
@@ -28,7 +40,7 @@ namespace Mono.TextEditor.Highlighting
 						break;
 					}
 					
-					if (doc[curOffset] != pattern[i]) {
+					if (doc [curOffset] != pattern [i]) {
 						match = false;
 						break;
 					}
