@@ -38,7 +38,6 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Core.ProgressMonitoring;
-using MonoDevelop.Projects.Dom.Output;
 using MonoDevelop.Projects.Policies;
 using MonoDevelop.Projects.Formats.MD1;
 using MonoDevelop.Projects.Extensions;
@@ -94,8 +93,8 @@ namespace MonoDevelop.Projects
 
 		public DotNetProject (string languageName, ProjectCreateInformation projectCreateInfo, XmlElement projectOptions) : this(languageName)
 		{
-			if ((projectOptions != null) && (projectOptions.Attributes["Target"] != null))
-				CompileTarget = (CompileTarget)Enum.Parse (typeof(CompileTarget), projectOptions.Attributes["Target"].Value);
+			if ((projectOptions != null) && (projectOptions.Attributes ["Target"] != null))
+				CompileTarget = (CompileTarget)Enum.Parse (typeof(CompileTarget), projectOptions.Attributes ["Target"].Value);
 			else if (IsLibraryBasedProjectType)
 				CompileTarget = CompileTarget.Library;
 
@@ -110,7 +109,7 @@ namespace MonoDevelop.Projects
 					if (!projectOptions.HasAttribute ("Platform")) {
 						// Clone the element since we are going to change it
 						platform = GetDefaultTargetPlatform (projectCreateInfo);
-						projectOptions = (XmlElement) projectOptions.CloneNode (true);
+						projectOptions = (XmlElement)projectOptions.CloneNode (true);
 						projectOptions.SetAttribute ("Platform", platform);
 					} else
 						platform = projectOptions.GetAttribute ("Platform");
@@ -126,9 +125,16 @@ namespace MonoDevelop.Projects
 				Configurations.Add (configDebug);
 
 				DotNetProjectConfiguration configRelease = CreateConfiguration ("Release" + platformSuffix) as DotNetProjectConfiguration;
-				if (projectOptions != null)
-					projectOptions.SetAttribute ("DefineDebug", "False");
-				configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (projectOptions);
+				
+				if (projectOptions != null) {
+					XmlElement releaseProjectOptions = (XmlElement)projectOptions.CloneNode (true);
+					releaseProjectOptions.SetAttribute ("Release", "True");
+					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (releaseProjectOptions);
+				} else {
+					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (null);
+				}
+				
+				configRelease.CompilationParameters.RemoveDefineSymbol ("DEBUG");
 				configRelease.DebugMode = false;
 				configRelease.ExternalConsole = externalConsole;
 				configRelease.PauseConsoleOutput = externalConsole;
@@ -136,15 +142,16 @@ namespace MonoDevelop.Projects
 			}
 
 			if ((projectOptions != null) && (projectOptions.Attributes["TargetFrameworkVersion"] != null))
-				targetFrameworkId = TargetFrameworkMoniker.Parse (projectOptions.Attributes["TargetFrameworkVersion"].Value);
+				newProjectTargetFrameworkId = TargetFrameworkMoniker.Parse (projectOptions.Attributes["TargetFrameworkVersion"].Value);
 
 			string binPath;
 			if (projectCreateInfo != null) {
 				Name = projectCreateInfo.ProjectName;
 				binPath = projectCreateInfo.BinPath;
 				defaultNamespace = SanitisePotentialNamespace (projectCreateInfo.ProjectName);
-			} else
+			} else {
 				binPath = ".";
+			}
 
 			foreach (DotNetProjectConfiguration dotNetProjectConfig in Configurations) {
 				dotNetProjectConfig.OutputDirectory = Path.Combine (binPath, dotNetProjectConfig.Name);
@@ -161,15 +168,6 @@ namespace MonoDevelop.Projects
 			get { return "DotNet"; }
 		}
 
-		private Ambience ambience;
-		public override Ambience Ambience {
-			get {
-				if (ambience == null)
-					ambience = AmbienceService.GetAmbienceForLanguage (LanguageName);
-				return ambience;
-			}
-		}
-
 		private string languageName;
 		public string LanguageName {
 			get { return languageName; }
@@ -234,7 +232,10 @@ namespace MonoDevelop.Projects
 					languageParameters.ParentProject = this;
 			}
 		}
-
+		
+		/// <summary>
+		/// Default namespace setting. May be empty, use GetDefaultNamespace to get a usable value.
+		/// </summary>
 		public string DefaultNamespace {
 			get { return defaultNamespace; }
 			set {
@@ -275,25 +276,30 @@ namespace MonoDevelop.Projects
 				return resourceHandler;
 			}
 		}
-
-		TargetFrameworkMoniker targetFrameworkId;
+		
+		//only used for initializing new projects
+		TargetFrameworkMoniker newProjectTargetFrameworkId;
+		
 		TargetFramework targetFramework;
 
 		public TargetFramework TargetFramework {
 			get {
-				SetDefaultFramework ();
+				if (targetFramework == null) {
+					var id = newProjectTargetFrameworkId ?? GetDefaultTargetFrameworkId ();
+					targetFramework = Runtime.SystemAssemblyService.GetTargetFramework (id);
+				}
 				return targetFramework;
 			}
 			set {
-				bool replacingValue = targetFramework != null;
-				TargetFramework validValue = GetValidFrameworkVersion (value);
-				if (targetFramework == null && validValue == null)
-					targetFramework = Services.ProjectService.DefaultTargetFramework;
-				if (targetFramework == validValue || validValue == null)
+				if (!SupportsFramework (value))
+					throw new ArgumentException ("Project does not support framework '" + value.Id.ToString () +"'");
+				if (value == null)
+					value = Runtime.SystemAssemblyService.GetTargetFramework (GetDefaultTargetFrameworkForFormat (FileFormat));
+				if (targetFramework != null && value.Id == targetFramework.Id)
 					return;
-				targetFramework = validValue;
-				targetFrameworkId = validValue.Id;
-				if (replacingValue)
+				bool updateReferences = targetFramework != null;
+				targetFramework = value;
+				if (updateReferences)
 					UpdateSystemReferences ();
 				NotifyModified ("TargetFramework");
 			}
@@ -303,11 +309,35 @@ namespace MonoDevelop.Projects
 			get { return Runtime.SystemAssemblyService.DefaultRuntime; }
 		}
 		
+		/// <summary>
+		/// Gets the target framework for new projects
+		/// </summary>
+		/// <returns>
+		/// The default target framework identifier.
+		/// </returns>
 		public virtual TargetFrameworkMoniker GetDefaultTargetFrameworkId ()
 		{
-			return ProjectService.DefaultTargetFrameworkId;
+			return Services.ProjectService.DefaultTargetFramework.Id;
 		}
-
+		
+		/// <summary>
+		/// Returns the default framework for a given format
+		/// </summary>
+		/// <returns>
+		/// The default target framework for the format.
+		/// </returns>
+		/// <param name='format'>
+		/// A format
+		/// </param>
+		/// <remarks>
+		/// This method is used to determine what's the correct target framework for a project
+		/// deserialized using a specific format.
+		/// </remarks>
+		public virtual TargetFrameworkMoniker GetDefaultTargetFrameworkForFormat (FileFormat format)
+		{
+			return GetDefaultTargetFrameworkId ();
+		}
+		
 		public IAssemblyContext AssemblyContext {
 			get {
 				if (composedAssemblyContext == null) {
@@ -328,16 +358,6 @@ namespace MonoDevelop.Projects
 			}
 		}
 
-		void SetDefaultFramework ()
-		{
-			if (targetFramework == null) {
-				if (targetFrameworkId != null)
-					targetFramework = Runtime.SystemAssemblyService.GetTargetFramework (targetFrameworkId);
-				if (targetFramework == null)
-					TargetFramework = Services.ProjectService.DefaultTargetFramework;
-			}
-		}
-
 		public virtual bool SupportsFramework (TargetFramework framework)
 		{
 			if (LanguageBinding == null)
@@ -352,34 +372,6 @@ namespace MonoDevelop.Projects
 			return false;
 		}
 
-		//if possible, find a ClrVersion that the language binding can handle
-		TargetFramework GetValidFrameworkVersion (TargetFramework suggestion)
-		{
-			if (suggestion == null) {
-				if (LanguageBinding == null)
-					return null;
-				else
-					suggestion = Services.ProjectService.DefaultTargetFramework;
-			}
-
-			if (SupportsFramework (suggestion))
-				return suggestion;
-
-			TargetFramework oneSupported = null;
-			foreach (TargetFramework f in Runtime.SystemAssemblyService.GetTargetFrameworks ()) {
-				if (SupportsFramework (f)) {
-					if (TargetRuntime.IsInstalled (f))
-						return f;
-					else if (oneSupported == null)
-						oneSupported = f;
-				}
-			}
-			if (oneSupported != null)
-				return oneSupported;
-
-			return null;
-		}
-
 		[ItemProperty(DefaultValue = true)]
 		public bool UsePartialTypes {
 			get { return usePartialTypes; }
@@ -463,16 +455,6 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		internal protected override BuildResult OnRunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration)
-		{
-			if (!TargetRuntime.IsInstalled (TargetFramework)) {
-				BuildResult res = new BuildResult ();
-				res.AddError (GettextCatalog.GetString ("Framework '{0}' not installed.", TargetFramework.Name));
-				return res;
-			}
-			return base.OnRunTarget (monitor, target, configuration);
-		}
-		
 		internal protected override void PopulateOutputFileList (List<FilePath> list, ConfigurationSelector configuration)
 		{
 			base.PopulateOutputFileList (list, configuration);
@@ -505,21 +487,28 @@ namespace MonoDevelop.Projects
 		}
 		
 		[ThreadStatic]
-		static int supportReferDistance = -1;
+		static int supportReferDistance;
+		[ThreadStatic]
+		static HashSet<DotNetProject> processedProjects;
 
 		internal protected override void PopulateSupportFileList (FileCopySet list, ConfigurationSelector configuration)
 		{
 			try {
+				if (supportReferDistance == 0)
+					processedProjects = new HashSet<DotNetProject> ();
 				supportReferDistance++;
+
 				PopulateSupportFileListInternal (list, configuration);
 			} finally {
 				supportReferDistance--;
+				if (supportReferDistance == 0)
+					processedProjects = null;
 			}
 		}
 
 		void PopulateSupportFileListInternal (FileCopySet list, ConfigurationSelector configuration)
 		{
-			if (supportReferDistance < 2)
+			if (supportReferDistance <= 2)
 				base.PopulateSupportFileList (list, configuration);
 
 			//rename the app.config file
@@ -534,10 +523,10 @@ namespace MonoDevelop.Projects
 			
 			//collect all the "local copy" references and their attendant files
 			foreach (ProjectReference projectReference in References) {
-				if (!projectReference.LocalCopy || ParentSolution == null)
+				if (!projectReference.LocalCopy || !projectReference.CanSetLocalCopy)
 					continue;
 
-				if (projectReference.ReferenceType == ReferenceType.Project) {
+				if (ParentSolution != null && projectReference.ReferenceType == ReferenceType.Project) {
 					DotNetProject p = ParentSolution.FindProjectByName (projectReference.Reference) as DotNetProject;
 
 					if (p == null) {
@@ -555,8 +544,9 @@ namespace MonoDevelop.Projects
 
 					//VS COMPAT: recursively copy references's "local copy" files
 					//but only copy the "copy to output" files from the immediate references
-					foreach (var f in p.GetSupportFileList (configuration))
-						list.Add (f.Src, f.CopyOnlyIfNewer, f.Target);
+					if (processedProjects.Add (p))
+						foreach (var f in p.GetSupportFileList (configuration))
+							list.Add (f.Src, f.CopyOnlyIfNewer, f.Target);
 
 					DotNetProjectConfiguration refConfig = p.GetConfiguration (configuration) as DotNetProjectConfiguration;
 
@@ -749,7 +739,8 @@ namespace MonoDevelop.Projects
 		{
 			// Make sure the fx version is sorted out before saving
 			// to avoid changes in project references while saving 
-			SetDefaultFramework ();
+			if (targetFramework == null)
+				targetFramework = Runtime.SystemAssemblyService.GetTargetFramework (GetDefaultTargetFrameworkForFormat (FileFormat));
 			base.OnSave (monitor);
 		}
 
@@ -797,6 +788,36 @@ namespace MonoDevelop.Projects
 			if (base.CheckNeedsBuild (configuration))
 				return true;
 			
+			// base.CheckNeedsBuild() checks Project references, but not Assembly, Package, or Custom.
+			DateTime mtime = GetLastBuildTime (configuration);
+			foreach (ProjectReference pref in References) {
+				switch (pref.ReferenceType) {
+				case ReferenceType.Assembly:
+					foreach (var file in GetAssemblyRefsRec (pref.Reference, new HashSet<string> ())) {
+						try {
+							if (File.GetLastWriteTime (file) > mtime)
+								return true;
+						} catch (IOException) {
+							// Ignore.
+						}
+					}
+					break;
+				case ReferenceType.Package:
+					if (pref.Package == null) {
+						break;
+					}
+					foreach (var assembly in pref.Package.Assemblies) {
+						try {
+							if (File.GetLastWriteTime (assembly.Location) > mtime)
+								return true;
+						} catch (IOException) {
+							// Ignore.
+						}
+					}
+					break;
+				}
+			}
+			
 			return Files.Any (file => file.BuildAction == BuildAction.EmbeddedResource
 					&& String.Compare (Path.GetExtension (file.FilePath), ".resx", StringComparison.OrdinalIgnoreCase) == 0
 					&& MD1DotNetProjectHandler.IsResgenRequired (file.FilePath));
@@ -839,7 +860,7 @@ namespace MonoDevelop.Projects
 			DotNetExecutionCommand cmd = new DotNetExecutionCommand (configuration.CompiledOutputName);
 			cmd.Arguments = configuration.CommandLineParameters;
 			cmd.WorkingDirectory = Path.GetDirectoryName (configuration.CompiledOutputName);
-			cmd.EnvironmentVariables = new Dictionary<string, string> (configuration.EnvironmentVariables);
+			cmd.EnvironmentVariables = configuration.GetParsedEnvironmentVariables ();
 			cmd.TargetRuntime = TargetRuntime;
 			cmd.UserAssemblyPaths = GetUserAssemblyPaths (configSel);
 			return cmd;
@@ -877,7 +898,11 @@ namespace MonoDevelop.Projects
 				return false;
 			return LanguageBinding.IsSourceCodeFile (fileName);
 		}
-
+		
+		/// <summary>
+		/// Gets the default namespace for the file, according to the naming policy.
+		/// </summary>
+		/// <remarks>Always returns a valid namespace, even if the fileName is null.</remarks>
 		public virtual string GetDefaultNamespace (string fileName)
 		{
 			DotNetNamingPolicy pol = Policies.Get<DotNetNamingPolicy> ();
@@ -887,6 +912,10 @@ namespace MonoDevelop.Projects
 			string defaultNmspc = !string.IsNullOrEmpty (DefaultNamespace)
 				? DefaultNamespace
 				: SanitisePotentialNamespace (Name) ?? "Application";
+			
+			if (string.IsNullOrEmpty (fileName)) {
+				return defaultNmspc;
+			}
 
 			string dirname = Path.GetDirectoryName (fileName);
 			string relativeDirname = null;
@@ -962,28 +991,18 @@ namespace MonoDevelop.Projects
 		// Make sure that the project references are valid for the target clr version.
 		void UpdateSystemReferences ()
 		{
-			ArrayList toDelete = new ArrayList ();
-			ArrayList toAdd = new ArrayList ();
-
 			foreach (ProjectReference pref in References) {
-				if (pref.ReferenceType == ReferenceType.Gac) {
+				if (pref.ReferenceType == ReferenceType.Package) {
 					string newRef = AssemblyContext.GetAssemblyNameForVersion (pref.Reference, pref.Package != null ? pref.Package.Name : null, this.TargetFramework);
 					if (newRef == null) {
 						pref.ResetReference ();
 					} else if (newRef != pref.Reference) {
-						toDelete.Add (pref);
-						toAdd.Add (new ProjectReference (ReferenceType.Gac, newRef));
+						pref.Reference = newRef;
 					} else if (!pref.IsValid) {
 						pref.ResetReference ();
 					}
 				}
 			}
-			foreach (ProjectReference pref in toDelete) {
-				References.Remove (pref);
-			}
-			foreach (ProjectReference pref in toAdd) {
-				References.Add (pref);
-			}
 		}
 
 		protected override IEnumerable<string> GetStandardBuildActions ()
@@ -1023,9 +1042,6 @@ namespace MonoDevelop.Projects
 			IResourceHandler handler = ItemHandler as IResourceHandler;
 			if (handler != null)
 				MigrateResourceIds (handler, ResourceHandler);
-			
-			if (String.IsNullOrEmpty (defaultNamespace))
-				defaultNamespace = SanitisePotentialNamespace (Name);
 
 			base.OnEndLoad ();
 		}
