@@ -37,7 +37,6 @@ using MonoDevelop;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom.Output;
 
 
 namespace MonoDevelop.Projects
@@ -140,13 +139,6 @@ namespace MonoDevelop.Projects
 		}
 		IconId stockIcon = "md-project";
 		
-		/// <summary>
-		/// Ambience to use to show information about the project
-		/// </summary>
-		public virtual Ambience Ambience {
-			get { return new NetAmbience (); }
-		}
-
 		/// <summary>
 		/// List of languages that this project supports
 		/// </summary>
@@ -393,6 +385,14 @@ namespace MonoDevelop.Projects
 			AddFile (newDir);
 			return newDir;
 		}
+		
+		//HACK: the build code is structured such that support file copying is in here instead of the item handler
+		//so in order to avoid doing them twice when using the msbuild engine, we special-case them
+		bool UsingMSBuildEngine ()
+		{
+			var msbuildHandler = ItemHandler as MonoDevelop.Projects.Formats.MSBuild.MSBuildProjectHandler;
+			return msbuildHandler != null && msbuildHandler.UseXbuild;
+		}
 
 		protected override BuildResult OnBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
@@ -403,6 +403,15 @@ namespace MonoDevelop.Projects
 				cres.AddError (GettextCatalog.GetString ("Configuration '{0}' not found in project '{1}'", configuration.ToString (), Name));
 				return cres;
 			}
+			
+			StringParserService.Properties["Project"] = Name;
+			
+			if (UsingMSBuildEngine ()) {
+				var r = DoBuild (monitor, configuration);
+				isDirty = false;
+				return r;
+			}
+			
 			string outputDir = conf.OutputDirectory;
 			try {
 				DirectoryInfo directoryInfo = new DirectoryInfo (outputDir);
@@ -415,10 +424,9 @@ namespace MonoDevelop.Projects
 
 			//copy references and files marked to "CopyToOutputDirectory"
 			CopySupportFiles (monitor, configuration);
-
-			StringParserService.Properties["Project"] = Name;
-
-			monitor.BeginTask (GettextCatalog.GetString ("Performing main compilation..."), 0);
+		
+			monitor.Log.WriteLine ("Performing main compilation...");
+			
 			BuildResult res = DoBuild (monitor, configuration);
 
 			isDirty = false;
@@ -429,7 +437,6 @@ namespace MonoDevelop.Projects
 
 				monitor.Log.WriteLine (GettextCatalog.GetString ("Build complete -- ") + errorString + ", " + warningString);
 			}
-			monitor.EndTask ();
 
 			return res;
 		}
@@ -452,8 +459,8 @@ namespace MonoDevelop.Projects
 			ProjectConfiguration config = (ProjectConfiguration) GetConfiguration (configuration);
 
 			foreach (FileCopySet.Item item in GetSupportFileList (configuration)) {
-				string dest = Path.GetFullPath (Path.Combine (config.OutputDirectory, item.Target));
-				string src = Path.GetFullPath (item.Src);
+				FilePath dest = Path.GetFullPath (Path.Combine (config.OutputDirectory, item.Target));
+				FilePath src = Path.GetFullPath (item.Src);
 
 				try {
 					if (dest == src)
@@ -462,10 +469,13 @@ namespace MonoDevelop.Projects
 					if (item.CopyOnlyIfNewer && File.Exists (dest) && (File.GetLastWriteTimeUtc (dest) >= File.GetLastWriteTimeUtc (src)))
 						continue;
 
+					// Use Directory.Create so we don't trigger the VersionControl addin and try to
+					// add the directory to version control.
 					if (!Directory.Exists (Path.GetDirectoryName (dest)))
-						FileService.CreateDirectory (Path.GetDirectoryName (dest));
+						Directory.CreateDirectory (Path.GetDirectoryName (dest));
 
 					if (File.Exists (src)) {
+						dest.Delete ();
 						FileService.CopyFile (src, dest);
 						
 						// Copied files can't be read-only, so they can be removed when rebuilding the project
@@ -500,16 +510,14 @@ namespace MonoDevelop.Projects
 			ProjectConfiguration config = (ProjectConfiguration) GetConfiguration (configuration);
 
 			foreach (FileCopySet.Item item in GetSupportFileList (configuration)) {
-				string dest = Path.Combine (config.OutputDirectory, item.Target);
+				FilePath dest = Path.Combine (config.OutputDirectory, item.Target);
 
 				// Ignore files which were not copied
 				if (Path.GetFullPath (dest) == Path.GetFullPath (item.Src))
 					continue;
 
 				try {
-					if (File.Exists (dest)) {
-						FileService.DeleteFile (dest);
-					}
+					dest.Delete ();
 				} catch (IOException ex) {
 					monitor.ReportError (GettextCatalog.GetString ("Error deleting support file '{0}'.", dest), ex);
 				}
@@ -531,8 +539,8 @@ namespace MonoDevelop.Projects
 		/// </remarks>
 		public FileCopySet GetSupportFileList (ConfigurationSelector configuration)
 		{
-			FileCopySet list = new FileCopySet ();
-			Services.ProjectService.GetExtensionChain (this).PopulateSupportFileList (this, list, configuration);
+			var list = new FileCopySet ();
+			PopulateSupportFileList (list, configuration);
 			return list;
 		}
 
@@ -574,7 +582,7 @@ namespace MonoDevelop.Projects
 		public List<FilePath> GetOutputFiles (ConfigurationSelector configuration)
 		{
 			List<FilePath> list = new List<FilePath> ();
-			Services.ProjectService.GetExtensionChain (this).PopulateOutputFileList (this, list, configuration);
+			PopulateOutputFileList (list, configuration);
 			return list;
 		}
 
@@ -623,25 +631,33 @@ namespace MonoDevelop.Projects
 		protected override void OnClean (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			SetDirty ();
+			
 			ProjectConfiguration config = GetConfiguration (configuration) as ProjectConfiguration;
 			if (config == null) {
 				monitor.ReportError (GettextCatalog.GetString ("Configuration '{0}' not found in project '{1}'", config.Id, Name), null);
 				return;
 			}
 			
-			// Delete generated files
+			if (UsingMSBuildEngine ()) {
+				DoClean (monitor, config.Selector);
+				return;
+			}
 			
+			monitor.Log.WriteLine ("Removing output files...");
+			
+			// Delete generated files
 			foreach (FilePath file in GetOutputFiles (configuration)) {
 				if (File.Exists (file)) {
-					FileService.DeleteFile (file);
+					file.Delete ();
 					if (file.ParentDirectory.CanonicalPath != config.OutputDirectory.CanonicalPath && Directory.GetFiles (file.ParentDirectory).Length == 0)
-						FileService.DeleteDirectory (file.ParentDirectory);
+						file.ParentDirectory.Delete ();
 				}
 			}
-
+	
 			DeleteSupportFiles (monitor, configuration);
-
+			
 			DoClean (monitor, config.Selector);
+			monitor.Log.WriteLine (GettextCatalog.GetString ("Clean complete"));
 		}
 
 		protected virtual void DoClean (IProgressMonitor monitor, ConfigurationSelector configuration)
