@@ -33,10 +33,9 @@ using System.Text;
 using System.Text.RegularExpressions;
 using System.Xml;
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Parser;
 using Mono.TextEditor;
 using Mono.TextEditor.PopupWindow;
+using MonoDevelop.Ide.CodeFormatting;
 
 namespace MonoDevelop.Ide.CodeTemplates
 {
@@ -64,11 +63,23 @@ namespace MonoDevelop.Ide.CodeTemplates
 			set; 
 		}
 		
+		string shortcut;
 		public string Shortcut {
-			get;
-			set;
+			get {
+				return shortcut;
+			}
+			set {
+				if (value != null) {
+					var trimmedValue = value.Trim ();
+					if (trimmedValue != value)
+						LoggingService.LogWarning ("Trimmed code template shortcut for:" + trimmedValue);
+					shortcut = trimmedValue;
+				} else {
+					shortcut = null;
+				}
+			}
 		}
-		
+
 		public CodeTemplateType CodeTemplateType {
 			get;
 			set;
@@ -145,20 +156,11 @@ namespace MonoDevelop.Ide.CodeTemplates
 			return start;
 		}
 		
-//		static string GetLeadingWhiteSpace (MonoDevelop.Ide.Gui.TextEditor editor, int lineNr)
-//		{
-//			string lineText = editor.GetLineText (lineNr);
-//			int index = 0;
-//			while (index < lineText.Length && Char.IsWhiteSpace (lineText[index]))
-//				index++;
-//			return index > 0 ? lineText.Substring (0, index) : "";
-//		}
-		
 		static Regex variableRegEx = new Regex ("\\$([^$]*)\\$", RegexOptions.Compiled);
 		
 		public List<string> ParseVariables (string code)
 		{
-			List<string> result = new List<string> ();
+			var result = new List<string> ();
 			foreach (Match match in variableRegEx.Matches (code)) {
 				string name = match.Groups[1].Value;
 				if (name == "end" || name == "selected" || string.IsNullOrEmpty (name) || name.Trim ().Length == 0)
@@ -171,7 +173,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 		
 		public void AddVariable (CodeTemplateVariable var)
 		{
-			this.variableDecarations.Add (var.Name, var);
+			variableDecarations.Add (var.Name, var);
 		}
 		
 		public class TemplateResult
@@ -205,15 +207,14 @@ namespace MonoDevelop.Ide.CodeTemplates
 		
 		public TemplateResult FillVariables (TemplateContext context)
 		{
-			ExpansionObject expansion = CodeTemplateService.GetExpansionObject (this);
-			TemplateResult result = new TemplateResult ();
-			StringBuilder sb = new StringBuilder ();
+			var expansion = CodeTemplateService.GetExpansionObject (this);
+			var result = new TemplateResult ();
+			var sb = new StringBuilder ();
 			int lastOffset = 0;
 			string code = context.Document.Editor.FormatString (context.InsertPosition, context.TemplateCode);
-			
 			result.TextLinks = new List<TextLink> ();
 			foreach (Match match in variableRegEx.Matches (code)) {
-				string name = match.Groups[1].Value;
+				string name = match.Groups [1].Value;
 				sb.Append (code.Substring (lastOffset, match.Index - lastOffset));
 				lastOffset = match.Index + match.Length;
 				if (string.IsNullOrEmpty (name)) { // $$ is interpreted as $
@@ -232,30 +233,30 @@ namespace MonoDevelop.Ide.CodeTemplates
 				TextLink link = result.TextLinks.Find (l => l.Name == name);
 				bool isNew = link == null;
 				if (isNew) {
-					link         = new TextLink (name);
-					if (!string.IsNullOrEmpty (variableDecarations[name].ToolTip))
-						link.Tooltip = GettextCatalog.GetString (variableDecarations[name].ToolTip);
-					link.Values  = new CodeTemplateListDataProvider (variableDecarations[name].Values);
-					if (!string.IsNullOrEmpty (variableDecarations[name].Function)) {
-						link.Values  = expansion.RunFunction (context, null, variableDecarations[name].Function);
+					link = new TextLink (name);
+					if (!string.IsNullOrEmpty (variableDecarations [name].ToolTip))
+						link.Tooltip = GettextCatalog.GetString (variableDecarations [name].ToolTip);
+					link.Values = new CodeTemplateListDataProvider (variableDecarations [name].Values);
+					if (!string.IsNullOrEmpty (variableDecarations [name].Function)) {
+						link.Values = expansion.RunFunction (context, null, variableDecarations [name].Function);
 					}
 					result.TextLinks.Add (link);
 				}
-				link.IsEditable = variableDecarations[name].IsEditable;
-				link.IsIdentifier = variableDecarations[name].IsIdentifier;
-				if (!string.IsNullOrEmpty (variableDecarations[name].Function)) {
-					IListDataProvider<string> functionResult = expansion.RunFunction (context, null, variableDecarations[name].Function);
+				link.IsEditable = variableDecarations [name].IsEditable;
+				link.IsIdentifier = variableDecarations [name].IsIdentifier;
+				if (!string.IsNullOrEmpty (variableDecarations [name].Function)) {
+					IListDataProvider<string > functionResult = expansion.RunFunction (context, null, variableDecarations [name].Function);
 					if (functionResult != null && functionResult.Count > 0) {
-						string s = (string)functionResult[functionResult.Count - 1];
+						string s = (string)functionResult [functionResult.Count - 1];
 						if (s == null) {
 							if (variableDecarations.ContainsKey (name)) 
-								s = variableDecarations[name].Default;
+								s = variableDecarations [name].Default;
 						}
 						if (s != null) {
-							link.AddLink (new Segment (sb.Length, s.Length));
+							link.AddLink (new TextSegment (sb.Length, s.Length));
 							if (isNew) {
 								link.GetStringFunc = delegate (Func<string, string> callback) {
-									return expansion.RunFunction (context, callback, variableDecarations[name].Function);
+									return expansion.RunFunction (context, callback, variableDecarations [name].Function);
 								};
 							}
 							sb.Append (s);
@@ -263,27 +264,48 @@ namespace MonoDevelop.Ide.CodeTemplates
 					} else {
 						AddDefaultValue (sb, link, name);
 					}
-				} else  {
+				} else {
 					AddDefaultValue (sb, link, name);
 				}
 			}
 			sb.Append (code.Substring (lastOffset, code.Length - lastOffset));
-			result.Code = sb.ToString ();
+			
+			// format & indent template code
+			var data = new TextEditorData ();
+			data.Text = sb.ToString ();
+			data.Document.TextReplaced += delegate(object sender, DocumentChangeEventArgs e) {
+				int delta = e.ChangeDelta;
+
+				foreach (var link in result.TextLinks) {
+					link.Links = new List<TextSegment> (link.Links.AdjustSegments (e));
+				}
+				if (result.CaretEndOffset > e.Offset)
+					result.CaretEndOffset += delta;
+			};
+			
+			var formatter = CodeFormatterService.GetFormatter (context.Document.Editor.Document.MimeType);
+			if (formatter != null && context.Document.HasProject) {
+				formatter.OnTheFlyFormat (context.Document, 0, data.Length);
+			}
+			
+			IndentCode (data, context.LineIndent);
+			result.Code = data.Text;
+			data.Dispose ();
 			return result;
 		}
 
 		void AddDefaultValue (StringBuilder sb, TextLink link, string name)
 		{
-			if (string.IsNullOrEmpty (variableDecarations[name].Default))
+			if (string.IsNullOrEmpty (variableDecarations [name].Default))
 				return;
-			link.AddLink (new Segment (sb.Length, variableDecarations[name].Default.Length));
+			link.AddLink (new TextSegment (sb.Length, variableDecarations[name].Default.Length));
 			sb.Append (variableDecarations[name].Default);
 		}
 
 		
 		public string IndentCode (string code, string eol, string indent)
 		{
-			StringBuilder result = new StringBuilder ();
+			var result = new StringBuilder ();
 			for (int i = 0; i < code.Length; i++) {
 				switch (code[i]) {
 				case '\r':
@@ -301,7 +323,15 @@ namespace MonoDevelop.Ide.CodeTemplates
 			}
 			return result.ToString ();
 		}
-		
+
+		static void IndentCode (TextEditorData data, string lineIndent)
+		{
+			for (int i = 1; i < data.LineCount; i++) {
+				var line = data.GetLine (i + 1);
+				if (line.Length > 0)
+					data.Insert (line.Offset, lineIndent);
+			}
+		}
 		
 		string GetIndent (StringBuilder sb)
 		{
@@ -310,7 +340,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 			while (i >= 0 && !Char.IsWhiteSpace (str[i])) {
 				i--;
 			}
-			StringBuilder indent = new StringBuilder ();
+			var indent = new StringBuilder ();
 			while (i >= 0 && (str[i] == ' ' || str[i] == '\t')) {
 				indent.Append (str[i]);
 				i--;
@@ -320,26 +350,26 @@ namespace MonoDevelop.Ide.CodeTemplates
 		
 		string RemoveIndent (string text, string indent)
 		{
-			Document doc = new Document ();
+			var doc = new TextDocument ();
 			doc.Text = text;
-			StringBuilder result = new StringBuilder ();
-			foreach (LineSegment line in doc.Lines) {
+			var result = new StringBuilder ();
+			foreach (DocumentLine line in doc.Lines) {
 				string curLineIndent = line.GetIndentation (doc);
 				int offset = Math.Min (curLineIndent.Length, indent.Length);
-				result.Append (doc.GetTextBetween (line.Offset + offset, line.EndOffset));
+				result.Append (doc.GetTextBetween (line.Offset + offset, line.EndOffsetIncludingDelimiter));
 			}
 			return result.ToString ();
 		}
 		
 		string Reindent (string text, string indent)
 		{
-			Document doc = new Document ();
+			var doc = new TextDocument ();
 			doc.Text = text;
-			StringBuilder result = new StringBuilder ();
-			foreach (LineSegment line in doc.Lines) {
+			var result = new StringBuilder ();
+			foreach (DocumentLine line in doc.Lines) {
 				if (result.Length > 0)
 					result.Append (indent);
-				result.Append (doc.GetTextAt (line));
+				result.Append (doc.GetTextAt (line.SegmentIncludingDelimiter));
 			}
 			return result.ToString ();
 		}
@@ -359,21 +389,18 @@ namespace MonoDevelop.Ide.CodeTemplates
 		/// </summary>
 		public TemplateResult InsertTemplateContents (MonoDevelop.Ide.Gui.Document document)
 		{
-			ProjectDom dom = document.Dom;
-			ParsedDocument doc = document.ParsedDocument ?? MonoDevelop.Projects.Dom.Parser.ProjectDomService.GetParsedDocument (dom, document.FileName);
 			Mono.TextEditor.TextEditorData data = document.Editor;
-
+			
 			int offset = data.Caret.Offset;
 //			string leadingWhiteSpace = GetLeadingWhiteSpace (editor, editor.CursorLine);
 			
-			TemplateContext context = new TemplateContext {
+			var context = new TemplateContext {
 				Template = this,
 				Document = document,
-				ProjectDom = dom,
-				ParsedDocument = doc,
+				ParsedDocument = document.ParsedDocument.ParsedFile,
 				InsertPosition = data.Caret.Location,
 				LineIndent = data.Document.GetLineIndent (data.Caret.Line),
-				TemplateCode = IndentCode (Code, document.Editor.EolMarker, data.Document.GetLineIndent (data.Caret.Line))
+				TemplateCode = Code
 			};
 
 			if (data.IsSomethingSelected) {
@@ -397,15 +424,18 @@ namespace MonoDevelop.Ide.CodeTemplates
 			
 			TemplateResult template = FillVariables (context);
 			template.InsertPosition = offset;
-			int length = document.Editor.Insert (offset, template.Code);
+			document.Editor.Insert (offset, template.Code);
 			
+			int newoffset;
 			if (template.CaretEndOffset >= 0) {
-				document.Editor.Caret.Offset = offset + template.CaretEndOffset; 
+				newoffset = offset + template.CaretEndOffset; 
 			} else {
-				document.Editor.Caret.Offset = offset + template.Code.Length; 
+				newoffset = offset + template.Code.Length; 
 			}
+
+			document.Editor.Caret.Location = document.Editor.OffsetToLocation (newoffset) ;
 			
-			if (PropertyService.Get ("OnTheFlyFormatting", false)) {
+/*			if (PropertyService.Get ("OnTheFlyFormatting", false)) {
 				string mt = DesktopService.GetMimeTypeForUri (document.FileName);
 				var formatter = MonoDevelop.Ide.CodeFormatting.CodeFormatterService.GetFormatter (mt);
 				if (formatter != null && formatter.SupportsOnTheFlyFormatting) {
@@ -414,7 +444,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 						document.Editor, offset, offset + length);
 					document.Editor.Document.EndAtomicUndo ();
 				}
-			}
+			}*/
 			return template;
 		}
 
@@ -434,15 +464,15 @@ namespace MonoDevelop.Ide.CodeTemplates
 		
 		const string CodeNode            = "Code";
 			
-		const string versionAttribute    = "version";
-		const string version             = "2.0";
+		const string VersionAttribute    = "version";
+		const string CurrentVersion      = "2.0";
 		
-		const string descriptionAttribute = "description";
+		const string DescriptionAttribute = "description";
 		
 		public void Write (XmlWriter writer)
 		{
 			writer.WriteStartElement (Node);
-			writer.WriteAttributeString (versionAttribute, version);
+			writer.WriteAttributeString (VersionAttribute, CurrentVersion);
 			
 			writer.WriteStartElement (HeaderNode);
 			
@@ -496,7 +526,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 		{
 			Debug.Assert (reader.LocalName == Node);
 			
-			CodeTemplate result = new CodeTemplate ();
+			var result = new CodeTemplate ();
 			
 			XmlReadHelper.ReadList (reader, Node, delegate () {
 				//Console.WriteLine (reader.LocalName);
@@ -534,7 +564,7 @@ namespace MonoDevelop.Ide.CodeTemplates
 						//Console.WriteLine ("var:" + reader.LocalName);
 						switch (reader.LocalName) {
 						case CodeTemplateVariable.Node:
-							CodeTemplateVariable var = CodeTemplateVariable.Read (reader);
+							var var = CodeTemplateVariable.Read (reader);
 							result.variableDecarations [var.Name] = var;
 							return true;
 						}
